import pandas as pd
from gekko import GEKKO




def calculate_stream_numbers(self):
    # Expressions for N_HS and N_CS
    self.N_HS = len(self.stream_list[self.stream_list['StreamType'] == 'HS'])
    self.N_CS = len(self.stream_list[self.stream_list['StreamType'] == 'CS'])
    # Determine the number of stages
    self.N_stage = max(self.N_CS, self.N_HS)

def hen_stream_list(self):
    flux_chaud = self.stream_list[self.stream_list['StreamType'] == 'HS']
    flux_froid = self.stream_list[self.stream_list['StreamType'] == 'CS']
    self.TiHS = flux_chaud['Ti'].tolist()
    self.ToHS = flux_chaud['To'].tolist()
    self.TiCS = flux_froid['Ti'].tolist()
    self.ToCS = flux_froid['To'].tolist()
    self.mCpHS=flux_chaud['mCp'].tolist()
    self.mCpCS=flux_froid['mCp'].tolist()
    self.nameHS=flux_chaud['name'].tolist()
    self.nameCS=flux_froid['name'].tolist()


###############################"GHE#############################################"""  
def HeatExchangerNetwork(self,disp=False,dTmin=10.0):
    # Initialiser le modèle Gekko
    m = GEKKO(remote=False)
    # Utiliser les données extraites par HeatExchangerNetwork
    N_HS = self.N_HS
    N_CS = self.N_CS
    N_stage = self.N_stage
    TiHS = self.TiHS
    ToHS = self.ToHS
    TiCS = self.TiCS
    ToCS = self.ToCS
    mCpHS = self.mCpHS
    mCpCS = self.mCpCS
    nameHS=self.nameHS
    nameCS=self.nameCS

    # Autres paramètres
    ToCU, ToHU, TiCU, TiHU = 25, 150, 20, 200
    CCU, CHU =100, 100
    CF=100 #coût lié au nombre d'échangeur)
    U = 1.0
    B = 0.6

    #Variables:
    # Redéfinition des variables q et z comme des listes tridimensionnelles
    q = [[[m.Var(lb=0, name=f'q_{i}_{j}_{k}') for k in range(N_stage+1)] for j in range(N_CS)] for i in range(N_HS)]
    # Définition de z en tant que variable binaire
    z = [[[m.Var(lb=0, ub=1, integer=True, name=f'z_{i}_{j}_{k}') for k in range(N_stage+1)] for j in range(N_CS)] for i in range(N_HS)]
    qcu = [m.Var(lb=0, name=f'qcu_{i}') for i in range(N_HS)]
    qhu = [m.Var(lb=0, name=f'qhu_{j}') for j in range(N_CS)]

    # Températures aux étages pour les flux chauds et froids
    tHS = [[m.Var(TiHS[i], lb=ToHS[i], ub=TiHS[i], name=f'tHS_{i}_{k}') for k in range(N_stage+1)] for i in range(N_HS)]
    tCS = [[m.Var(TiCS[j], lb=TiCS[j], ub=ToCS[j], name=f'tCS_{j}_{k}') for k in range(N_stage+1)] for j in range(N_CS)]

    # Variables dt
    dtcu = [m.Var(value=0.0, lb=0.0) for i in range(N_HS)]
    dthu = [m.Var(value=0.0, lb=0.0) for j in range(N_CS)]
    dt = [[[m.Var(lb=0, name=f'dt_{i}_{j}_{k}') for k in range(N_stage+1)] for j in range(N_CS)] for i in range(N_HS)]

    # Contraintes pour fixer les températures d'entrée
    for i in range(N_HS):
        m.Equation(tHS[i][0] == TiHS[i])
    for j in range(N_CS):
        m.Equation(tCS[j][N_stage] == TiCS[j])

    # Définition de z en fonction de q
    for k in range(N_stage+1):
        for i in range(N_HS):
            for j in range(N_CS):
                m.Equation(z[i][j][k] == 1 - m.if3(q[i][j][k], 1, 0))
                #m.Equation(z[i][j][k]<1)

    for k in range(N_stage+1):
        for i in range(N_HS):
            for j in range(N_CS):
                m.Equation(dt[i][j][k] <= (tHS[i][k] - tCS[j][k])+1000000.0*(1-z[i][j][k]))
                m.Equation(dt[i][j][k] >=dTmin)
        

    for k in range(N_stage):
        for i in range(N_HS):
            for j in range(N_CS):
                m.Equation(dt[i][j][k+1] <= (tHS[i][k+1] - tCS[j][k+1])+1000000.0*(1-z[i][j][k]))
                m.Equation(dt[i][j][k] >=dTmin)

    # Bilan d'énergie à chaque étage
    for i in range(N_HS):
        for k in range(N_stage):
            m.Equation((tHS[i][k] - tHS[i][k+1]) * mCpHS[i] == sum(q[i][j][k] for j in range(N_CS)))
    for j in range(N_CS):
        for k in range(N_stage):
            m.Equation((tCS[j][k] - tCS[j][k+1]) * mCpCS[j] == sum(q[i][j][k] for i in range(N_HS)))
            

    # Bilan d'énergie global pour chaque flux
    for i in range(N_HS):
        m.Equation((TiHS[i] - ToHS[i]) * mCpHS[i] == sum(q[i][j][k] for j in range(N_CS) for k in range(N_stage+1)) + qcu[i])
    for j in range(N_CS):
        m.Equation((ToCS[j] - TiCS[j]) * mCpCS[j] == sum(q[i][j][k] for i in range(N_HS) for k in range(N_stage+1)) + qhu[j])

    # Contraintes pour les utilités froides et chaudes

    for i in range(N_HS):
        m.Equation((tHS[i][N_stage] - ToHS[i]) * mCpHS[i] == qcu[i])
    for j in range(N_CS):
        m.Equation((ToCS[j] - tCS[j][0]) * mCpCS[j] == qhu[j])



    # Fonction objectif pour minimiser la somme de qcu et qhu
    cost = sum(CCU*qcu[i] for i in range(N_HS)) + sum(CHU*qhu[j] for j in range(N_CS))-0.0*sum(CF * z[i][j][k] for i in range(N_HS) for j in range(N_CS) for k in range(1, N_stage+1))

    m.Minimize(cost)

    # Résolution
    m.solve(disp=disp)
    #m.solve(disp=True, solver='ipopt')

    #calcul de mCpHS_ijk
    mCpCS_ijk = [[[0 for k in range(N_stage)] for j in range(N_CS)] for i in range(N_HS)]
    mCpHS_ijk = [[[0 for k in range(N_stage)] for j in range(N_CS)] for i in range(N_HS)]

    for i in range(N_HS):
        for j in range(N_CS):
            for k in range(N_stage):
                if tCS[j][k].value[0]>tCS[j][k+1].value[0]:
                    mCpCS_ijk[i][j][k]=q[i][j][k].value[0]/(tCS[j][k].value[0]-tCS[j][k+1].value[0])
                else:
                    mCpCS_ijk[i][j][k]=0
                
                if tHS[i][k].value[0]>tHS[i][k+1].value[0]:
                    mCpHS_ijk[i][j][k]=q[i][j][k].value[0]/(tHS[i][k].value[0]-tHS[i][k+1].value[0])
                else:
                    mCpHS_ijk[i][j][k]=0

                #print(f'mCpCS_ijk[{i}][{j}][{k}]={mCpCS_ijk[i][j][k]}')
                #print(f'mCpHS_ijk[{i}][{j}][{k}]={mCpHS_ijk[i][j][k]}')

    ############################"""""" Affichage des résultats############################

    # Créer des listes pour stocker les valeurs de q et z ainsi que les indices i, j, k
    q_values = []
    i_values = []
    j_values = []
    k_values = []
    mCpCS_values=[]
    mCpHS_values=[]
    nameCS_values=[]
    nameHS_values=[]
    TiCS_values=[]
    TiHS_values=[]
    ToCS_values=[]
    ToHS_values=[]

    for i in range(N_HS):
        for j in range(N_CS):
            for k in range(N_stage):
                mCpCS_value=mCpCS_ijk[i][j][k]
                mCpHS_value=mCpHS_ijk[i][j][k]
                q_value = round(q[i][j][k].value[0],3)
                q_values.append(q_value)
                i_values.append(i)
                j_values.append(j)
                k_values.append(k)

                mCpCS_values.append(mCpCS_value)
                mCpHS_values.append(mCpHS_value)
                nameCS_values.append(nameCS[j])
                nameHS_values.append(nameHS[i])
                ToCS_values.append(round(tCS[j][k].value[0],1))
                TiHS_values.append(round(tHS[i][k].value[0],1))
                
                TiCS_values.append(round(tCS[j][k+1].value[0],1))
                ToHS_values.append(round(tHS[i][k+1].value[0],1))
    




    # Créer un DataFrame pour q avec les valeurs et les indices i, j, k correspondants
    self.hen_results = pd.DataFrame({'stage k': k_values, 'HS name':nameHS_values, 'CS name':nameCS_values,'q(kW)': q_values,'mCpCS(kW/K)': mCpCS_values,'Ti_CS(°C)':TiCS_values,'To_CS(°C)':ToCS_values,'mCpHS(kW/K)': mCpHS_values,'Ti_HS(°C)':TiHS_values,'To_HS(°C)':ToHS_values})
                # Créer un DataFrame pour les utilités froides (qcu) et chaudes (qhu)
    qcu_values = [round(qcu[i].value[0],3) for i in range(N_HS)]
    qhu_values = [round(qhu[j].value[0],3) for j in range(N_CS)]

    #print("\nBilan d'énergie pour les utilités froides (qcu):", qcu_total)
    #print("Bilan d'énergie pour les utilités chaudes (qhu):", qhu_total)

    self.hen_qcu = pd.DataFrame({'qcu_values': qcu_values,'nameHS':nameHS})
    self.hen_qhu = pd.DataFrame({'qhu_values': qhu_values,'nameCS':nameCS})

    # Calculer les bilans d'énergie
    self.hen_qcu_total = sum(qcu_values)
    self.hen_qhu_total = sum(qhu_values)

    ###########Supprimer les lignes vides ##################
    # Supprimer les lignes où qcu est égal à zéro
    self.hen_results = self.hen_results[self.hen_results['q(kW)'] != 0]

    # Supprimer les lignes où qcu_values est égal à zéro
    self.hen_qcu = self.hen_qcu[self.hen_qcu['qcu_values'] != 0]

    # Supprimer les lignes où qhu_values est égal à zéro
    self.hen_qhu = self.hen_qhu[self.hen_qhu['qhu_values'] != 0]



##################"print#######################"""""
    for i in range(N_HS):
        for j in range(N_CS):
            for k in range(N_stage):
                pass
                # print(f'Chaleur échangée q[{i}][{j}][{k}]: {q[i][j][k].value[0]}')  
                # print(f'z[{i}][{j}][{k}]: {z[i][j][k].value[0]}')     
                # print(f'qCS[i][j][k]: ({tCS[j][k].value[0]}-{tCS[j][k+1].value[0]})*{mCpCS[j]}={q[i][j][k].value[0]}')
                # print(f'qHS[i][j][k]: ({tHS[i][k].value[0]}-{tHS[i][k+1].value[0]})*{mCpHS[i]}={q[i][j][k].value[0]}')

    for i in range(N_HS):
        for j in range(N_CS):
            for k in range(N_stage+1):
                print(f'pinch dt[{i}][{j}][{k}]: {dt[i][j][k].value[0]}')



    for i in range(N_HS):
        #print(f'Températures aux étages pour le flux chaud {i}:')
        for k in range(N_stage+1):
            pass
            #print(f'tHS_{i}_{k}: {tHS[i][k].value[0]}')

    for j in range(N_CS):
        #print(f'Températures aux étages pour le flux froid {j}:')
        for k in range(N_stage+1):
            pass
            #print(f'tCS_{j}_{k}: {tCS[j][k].value[0]}')



from PyQt5.QtWidgets import *

import os
import sys

# Ajout du chemin vers le dossier parent pour importer les modules
sys.path.insert(0, os.path.join(os.path.dirname(__file__), "..", ".."))

# Import du module CalculatorWindow depuis PyqtSimulator
from PyqtSimulator.calc_window import CalculatorWindow

# Initialisation de l'application PyQt
app = QApplication(sys.argv)

# Définition du style de l'application
app.setStyle('Fusion')

# Création de la fenêtre de la calculatrice
wnd = CalculatorWindow()

# Affichage de la fenêtre
wnd.show()

# Exécution de l'application
sys.exit(app.exec_())



from ThermodynamicCycles.FluidPort.FluidPort import FluidPort
from CoolProp.CoolProp import PropsSI
import pandas as pd 
import math
import matplotlib.pyplot as plt
import numpy as np

class Object :
    def __init__(self):

        self.Timestamp=None

        self.Inlet=FluidPort() 
        self.Outlet=FluidPort()
        self.Outlet.callback = self.on_pressure_change  # Callback pour changement de pression sortie
        self.Inlet.callback = self.on_inlet_pressure_change  # Callback pour changement de pression entrée
        self.df = []
       

        #Paramètres 
        self.Ti = None  # Modelica.Units.SI.Temperature Ti
        self.To = None  # Modelica.Units.SI.Temperature To
        #self.Inlet.fluid = None  # replaceable package fluidEau = Package_thermal.Toolbox.Media.MyThBlD.H2O
        self.roughness = True # Choice of considering surface roughness
        #   //Valeurs usuelles indices de rugosite (K) en mm
        #   //Nature de la surface interieure , Indice rugosite k
        #   //cuivre, plomb, laiton, inox  0,001 a 0,002
        #   //Tube PVC                     0,0015
        #   //Acier inox                   0,015
        #   //tube acier du commerce       0,045 a 0,09
        #   //Acier etire                  0,015
        #   //Acier soude                  0,045
        #   //acier galvanise              0,15
        #   //Acier rouille                0,1 a 1
        #   //fonte neuve                  0,25 a 0,8
        #   //fonte usagee                 0,8 a 1,5
        #   //fonte incrustee              1,5 a 2,5
        #   //tôle ou fonte asphaltee      0,01 a 0,015
        #   //ciment bien lisse            0,3
        #   //Beton ordinaire              1
        #   //beton grossier               5
        #   //bois bien rabote             5
        #   //bois ordinaire               1

        self.d_hyd = 0.04  # Modelica.Units.SI.Diameter d_hyd=0.04 Hydraulic diameter
        self.L = 1  # Modelica.Units.SI.Length L=1 Length
        self.K = 0.0015 * 10**(-3)  # Modelica.Units.SI.Length K=0.0015*10^(-3) Roughness (average height of surface asperities)
        self.alpha = math.pi / 2  # Modelica.Units.SI.Angle alpha=pi/2 inclinaison du tube par rapport a l horizontale
        self.delta_Z = None  # Modelica.Units.SI.Height delta_Z hauteur du tuyau
        self.delta_H = None  # Modelica.Units.SI.Height delta_H perte de pression en m
        self.eta = None  # Modelica.Units.SI.DynamicViscosity eta Dynamic viscosity of fluid
        self.rho = 998.2  # Modelica.Units.SI.Density rho(start=998.2) Density of fluid
        self.delta_P = 80  # Modelica.Units.SI.Pressure delta_P(start=80) perte de pression du aux frottements
        self.diff_P = None  # Modelica.Units.SI.Pressure diff_P dif de pression entre Inlet et Outlet
        self.IN_con = None  # Package_thermal.PressureLoss.StraightPipe.dp_overall_IN_con IN_con
        self.IN_var = None # Modelica.Fluid.Dissipation.PressureLoss.StraightPipe.dp_overall_IN_var IN_var
        self.m_flow = 1  # Modelica.Units.SI.MassFlowRate m_flow(start=1)
        self.perimeter = None
        self.A = None  # Real A(unit="m2") section du tube
        self.V = None  # Real V(unit="m/s")
        self.Re = None  # Real Re
        self.h = 10000  # Real h( start=10000);
        
   

    def on_pressure_change(self):
        """Callback appelé lorsque self.Outlet.P change."""
        print("Détection d'un changement de Outlet.P. Recalcul en cours...")
        self.calculate()

    def on_inlet_pressure_change(self):
        """Appelée automatiquement quand Inlet.P change (remontée de circuit aval)"""
        try:
            if hasattr(self, 'Inlet') and hasattr(self, 'Outlet'):
                if self.Inlet.P is not None and self.Outlet.P is None:
                    # Mode direct : P_entrée connue, calcul P_sortie
                    print(f"StraightPipe: P_entrée détectée {self.Inlet.P:.3f} bar → calcul P_sortie")
                    self.calculate()
                elif self.Inlet.P is not None and self.Outlet.P is not None:
                    # Mode vérification : recalculer P_sortie avec nouvelle P_entrée
                    old_p_out = self.Outlet.P
                    self.Outlet.P = None  # Forcer recalcul
                    self.calculate()
                    print(f"StraightPipe: P_entrée mise à jour {self.Inlet.P:.3f} bar → P_sortie recalculée {self.Outlet.P:.3f} bar (ancienne: {old_p_out:.3f})")
        except Exception as e:
            print(f"Erreur dans on_inlet_pressure_change: {e}")

    
    def calculate(self):
        self.m=self.Inlet.F
        self.Ti_degC=-273.15+PropsSI('T','P',self.Inlet.P,'H',self.Inlet.h,self.Inlet.fluid)
        
        # Potentiel (Inlet.h - Outlet.h = 0)  (pas traduit car cela semble être une équation d'énergie)
        self.h = self.Inlet.h
        # isenthalpic state transformation (no storage and no loss of energy)
        self.Outlet.h = self.Inlet.h

        self.rho = PropsSI('D', 'P', self.Inlet.P, 'H', self.Inlet.h, self.Inlet.fluid)
        #print( "self.rho", self.rho)
        self.eta = PropsSI('V', 'P', self.Inlet.P, 'H', self.Inlet.h, self.Inlet.fluid)
    
        #print("self.eta",self.eta)
        self.Ti = PropsSI('T', 'P', self.Inlet.P, 'H', self.Inlet.h, self.Inlet.fluid)
        #self.To = PropsSI('T', 'P', self.Outlet.P, 'H', self.Outlet.h, self.Inlet.fluid)


        self.delta_Z = self.L * math.sin(self.alpha)  # geom et autre

        self.A = (math.pi * self.d_hyd**2) / 4
        self.V = self.Inlet.F / (self.rho * self.A)

        self.Re = self.calculate_reynolds_number(self.rho,self.V,self.d_hyd, self.eta)

        self.delta_P=self.calculate_pressure_loss(self.L, self.d_hyd, self.rho, self.V, self.Re, self.roughness, self.K)



        self.Outlet.F=self.Inlet.F
        
        # Gérer le mode direct (P_entrée connue) ou inverse (P_sortie connue)
        if self.Outlet.P is not None:
            # Mode inverse : P_sortie connue → calcul P_entrée
            self.Inlet.P=self.delta_P+self.Outlet.P
            print("Mode inverse - self.Outlet.P",self.Outlet.P)
        else:
            # Mode direct : P_entrée connue → calcul P_sortie
            if self.Inlet.P is not None:
                self.Outlet.P=self.Inlet.P-self.delta_P
                print("Mode direct - self.Inlet.P",self.Inlet.P)
            else:
                print("Erreur : ni P_entrée ni P_sortie ne sont définies")
        
        self.Outlet.fluid=self.Inlet.fluid
            ##################"fonctions"

        self.df = pd.DataFrame({'StraightPipe': [self.Timestamp,self.Inlet.fluid,round(self.Ti_degC,1),round(self.Inlet.F,3),round(self.Inlet.h,0),round(self.Outlet.h,0),round(self.A,3),round(self.V,3),round(self.Re,0),round(self.delta_P,1),round(self.Inlet.P,0),round(self.Outlet.P,0)], },
                      index = ['Timestamp','fluid','Ti_degC','Inlet.F (kg/s)','Inlet.h (j/kg)','Outlet.h (j/kg)','A (m2)','V (m/s)','Re','delta_P(Pa)'
                               ,'Inlet.P(Pa)','Outlet.P(Pa)'])
        
    def calculate_reynolds_number(self,rho, V,d_hyd, eta):
        # Nombre de Reynolds
        Re = (rho*V*d_hyd)/eta
        #print("Re = (rho*V*d_hyd)/eta =","(",rho,"*",V,"*",d_hyd,")/",eta)
        return Re
    
    def calculate_pressure_loss(self,L, d_hyd, rho, velocity, Reynolds_number, roughness, K):
        k = K/d_hyd
        #print("Reynolds_number:", Reynolds_number)

        Re_lam_min = 1e3  # Minimum Reynolds number for laminar regime
        Re_lam_max = 2090 * (1 / max(0.007, k)) ** 0.0635  # Maximum Reynolds number for laminar regime
        Re_turb_min = 4000
        Re_lam_leave = min(Re_lam_max, max(Re_lam_min, 754 * math.exp(0.0065 / 0.007 if k <= 0.007 else 0.0065 / k)))

        if roughness == "Neglected":
            lambda_FRI = 0.3164 * Reynolds_number ** (-0.25)
        else:
            lambda_FRI = 0.25 * (max(Reynolds_number, Re_lam_leave) / (math.log10(k/3.7 + 5.74 / max(Reynolds_number, Re_lam_leave) ** 0.9))) ** 2

        if Reynolds_number < Re_lam_leave:
            lambda_FRI_cal = 64 / Reynolds_number
        elif Reynolds_number > Re_turb_min:
            lambda_FRI_cal = lambda_FRI / (Reynolds_number ** 2)
        else:
            # You'll need to implement the cubic interpolation function here
            lambda_FRI_cal = self.cubic_interpolation_lambda(Reynolds_number, Re_lam_leave, Re_turb_min, k) / Reynolds_number ** 2

        #print("lambda_FRI:", lambda_FRI)

        # Calculate pressure loss using the formula
        dp = lambda_FRI_cal * (L / d_hyd) * (rho / 2) * velocity ** 2

        return dp

    import math

    def cubic_interpolation_lambda(self,Re, Re1, Re2, k):
        # Point x1=lg(Re1) with derivative yd1=1 at y1=lg(lambda2(Re1))
        x1 = math.log10(Re1)
        y1 = math.log10(64 * Re1)
        yd1 = 1

        # Point x2=lg(Re2) with derivative yd2 at y2=lg(lambda2(Re2))
        aux1 = (0.5 / math.log(10)) * 5.74 * 0.9
        aux2 = k / 3.7 + 5.74 / Re2**0.9
        aux3 = math.log10(aux2)
        L2 = 0.25 * (Re2 / aux3)**2
        aux4 = 2.51 / math.sqrt(L2) + 0.27 * k
        aux5 = -2 * math.sqrt(L2) * math.log10(aux4)
        x2 = math.log10(Re2)
        y2 = math.log10(L2)
        yd2 = 2 + 4 * aux1 / (aux2 * aux3 * (Re2)**0.9)

        # Constants: Cubic polynomial between x1=lg(Re1) and x2=lg(Re2)
        diff_x = x2 - x1
        m = (y2 - y1) / diff_x
        c2 = (3 * m - 2 * yd1 - yd2) / diff_x
        c3 = (yd1 + yd2 - 2 * m) / (diff_x * diff_x)
        dx = math.log10(Re / Re1)

        # Calculate lambda2 using the cubic Hermite spline interpolation formula
        lambda2 = 64 * Re1 * (Re / Re1)**(yd1 + dx * (c2 + dx * c3))
        
        return lambda2

    def Plot(self):
        """
        Trace le point de fonctionnement du tuyau droit sur la courbe de réseau.
        La courbe de réseau montre delta_P (Pa) en fonction du débit (m3/h)
        pour des vitesses d'écoulement de 0 à 3 m/s basées sur le diamètre hydraulique.
        """
        if self.d_hyd is None or self.L is None:
            raise ValueError("Le diamètre hydraulique (d_hyd) et la longueur (L) doivent être définis avant de tracer le graphique")
        
        if self.delta_P is None or self.V is None:
            raise ValueError("Calculer d'abord le point de fonctionnement avec calculate()")
        
        # Section de passage en m²
        section = (math.pi * self.d_hyd ** 2) / 4
        
        # Générer la courbe de réseau pour des vitesses de 0 à 3 m/s
        vitesses = np.linspace(0.01, 3, 100)  # Vitesses de 0.01 à 3 m/s (éviter 0 pour éviter division par zéro)
        debits_reseau = vitesses * section * 3600  # Débit en m3/h
        
        # Calculer les pertes de charge pour chaque débit
        delta_p_reseau = []
        for v in vitesses:
            # Calculer le nombre de Reynolds pour cette vitesse
            Re = self.calculate_reynolds_number(self.rho, v, self.d_hyd, self.eta)
            # Calculer la perte de charge pour cette vitesse
            dp = self.calculate_pressure_loss(self.L, self.d_hyd, self.rho, v, Re, self.roughness, self.K)
            delta_p_reseau.append(dp)
        
        delta_p_reseau = np.array(delta_p_reseau)
        
        # Point de fonctionnement actuel
        q_actuel = self.Inlet.F / self.rho * 3600  # Débit volumique en m3/h
        delta_p_actuel = self.delta_P
        
        # Créer le graphique
        fig, ax1 = plt.subplots(figsize=(10, 6))
        
        # Tracer la courbe de réseau sur l'axe principal (kPa)
        ax1.plot(debits_reseau, delta_p_reseau / 1000, 'b-', linewidth=2, 
                label=f'Courbe de réseau DN{int(self.d_hyd*1000)} (L={self.L}m, K={self.K*1000:.4f}mm, Re={self.Re:.0f})')
        
        # Tracer le point de fonctionnement
        ax1.plot(q_actuel, delta_p_actuel / 1000, 'ro', markersize=10, 
                label=f'Point de fonctionnement ({q_actuel:.2f} m³/h, {delta_p_actuel/1000:.2f} kPa)')
        
        # Ajouter des lignes de référence pour les vitesses
        vitesses_ref = [0.5, 1.0, 1.5, 2.0, 2.5, 3.0]
        for v in vitesses_ref:
            q_ref = v * section * 3600
            if q_ref <= max(debits_reseau):
                ax1.axvline(x=q_ref, color='gray', linestyle='--', alpha=0.3)
                # Placer le texte au 1/3 de la hauteur du graphique
                y_pos = ax1.get_ylim()[0] + (ax1.get_ylim()[1] - ax1.get_ylim()[0]) * 0.33
                ax1.text(q_ref, y_pos, f'{v} m/s', rotation=90, 
                        verticalalignment='center', fontsize=8, color='gray')
        
        # Configuration de l'axe principal (kPa)
        ax1.set_xlabel('Débit (m³/h)', fontsize=12)
        ax1.set_ylabel('Perte de charge (kPa)', fontsize=12, color='b')
        ax1.tick_params(axis='y', labelcolor='b')
        
        # Déterminer le régime d'écoulement
        if self.Re < 2300:
            regime = "Laminaire"
        elif self.Re > 4000:
            regime = "Turbulent"
        else:
            regime = "Transitoire"
        
        ax1.set_title(f'Courbe de réseau - Tuyau droit DN{int(self.d_hyd*1000)} ({regime})', 
                     fontsize=14, fontweight='bold')
        ax1.grid(True, alpha=0.3)
        ax1.legend(loc='upper left', fontsize=10)
        
        # Créer un axe secondaire (bar)
        ax2 = ax1.twinx()
        ax2.set_ylabel('Perte de charge (bar)', fontsize=12, color='r')
        ax2.tick_params(axis='y', labelcolor='r')
        
        # Synchroniser les limites des deux axes (1 bar = 100 kPa)
        y1_min, y1_max = ax1.get_ylim()
        ax2.set_ylim(y1_min / 100, y1_max / 100)
        
        # Afficher les informations techniques en bas à droite
        info_text = (f'DN: {int(self.d_hyd*1000)} mm\n'
                    f'Longueur: {self.L} m\n'
                    f'Section: {section*10000:.2f} cm²\n'
                    f'Rugosité: {self.K*1000:.4f} mm\n'
                    f'Vitesse actuelle: {self.V:.2f} m/s\n'
                    f'Reynolds: {self.Re:.0f} ({regime})')
        ax1.text(0.98, 0.02, info_text, transform=ax1.transAxes, 
                fontsize=9, verticalalignment='bottom', horizontalalignment='right',
                bbox=dict(boxstyle='round', facecolor='wheat', alpha=0.5))
        
        fig.tight_layout()
        plt.show()
        
        return fig



from OpenWeatherMap import get_weather
#import get_weather
#pip install requests[security]

def API_call():

# 
    api_key = get_weather.get_api_key()
#    print(api_key)
    
    location=get_weather.get_location()
#    print(location[0])

    weather = get_weather.get_weather(api_key, location[0],location[1],location[2])
# 
    #print(weather['main']['temp']-273.15,"°C",weather['main']['humidity'],"%")
   # print(weather)
    T=weather['main']['temp']-273.15
    RH=weather['main']['humidity']
    return T,RH

#API_T_RH=API_call()
#print(API_T_RH[0])
 
# if __name__ == '__main__':
#     main()


from PyQt5.QtCore import *
from PyqtSimulator.calc_conf import *
from PyqtSimulator.calc_node_base import *
from NodeEditor.nodeeditor.utils import dumpException

from CoolProp.CoolProp import PropsSI

class CalcInputContent(QDMNodeContentWidget):
    def initUI(self):
        
        
        
        self.fluid_lbl = QLabel("Type de fluide", self)
        self.fluid=QComboBox(self)
        
        self.fluid.addItem("ammonia")
        self.fluid.addItem("water")
        listeDesFluides=Fluid().listeDesFluides()
        
        for i in listeDesFluides:
            self.fluid.addItem(i)        
        # self.fluid.setStyleSheet("QComboBox"
        #                              "{"
        #                               "background-color: lightgreen;"
        #                              "}") 
        self.fluid.view().setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)                   
        
        self.F_kgs_lbl=QLabel("débit (kg/s)", self) 
        self.F_kgs_edit=QLineEdit("0.27777777", self)
        
        self.h_lbl = QLabel("enthalpie (kJ/kg-K)", self)              
        self.h_edit = QLineEdit("500", self)
        
        self.P_lbl = QLabel("Pression (bar)", self)                       
        self.P_edit = QLineEdit("1.01325", self)
        
        self.layout=QVBoxLayout()
        
        self.layout.addWidget(self.fluid_lbl)
        self.layout.addWidget(self.fluid)        
        self.layout.addWidget(self.F_kgs_lbl)
        self.layout.addWidget(self.F_kgs_edit)        
        self.layout.addWidget(self.h_lbl)
        self.layout.addWidget(self.h_edit)
        self.layout.addWidget(self.P_lbl)
        self.layout.addWidget(self.P_edit)
        
        
        
        
        self.setLayout(self.layout)
        
        self.layout.setAlignment(Qt.AlignRight)
        self.layout.setObjectName(self.node.content_label_objname)
        

    def serialize(self):
        res = super().serialize()
        res['h'] = self.h_edit.text()
        res2 = super().serialize()
        res2['P'] = self.P_edit.text()
        
        res3 = super().serialize()
        res3['fluid'] = self.fluid.currentText()
        
        res4 = super().serialize()
        res4['F'] = self.F_kgs_edit.text()
        
        return res,res2,res3,res4

    def deserialize(self, data, hashmap={}):
        res = super().deserialize(data, hashmap)
        res2 = super().deserialize(data, hashmap)
        res3 = super().deserialize(data, hashmap)
        res4 = super().deserialize(data, hashmap)
       # print("res=",res,res2,res3,res4)
       # print("dataaaaaaaaaa=",data)
        try:
            
            h = data[0]["h"]
            P = data[1]['P']
            fluid = data[2]['fluid']
            F = data[3]['F']
            
           # print("values=",h,P,fluid,F)
            
            self.h_edit.setText(h)
            self.P_edit.setText(P)
            self.fluid.setCurrentText(fluid)
            self.F_kgs_edit.setText(F)
            
            return True & res & res2 & res3 & res4
        except Exception as e:
            dumpException(e)
        return res,res2,res3,res4


@register_node(OP_NODE_INPUT_P_h)
class CalcNode_Input(CalcNode):
    icon = "icons/in.png"
    op_code = OP_NODE_INPUT_P_h
    op_title = "Source_P_h"
    content_label_objname = "calc_node_input"
    

    def __init__(self, scene):
        super().__init__(scene, inputs=[], outputs=[3])
        self.eval()

    def initInnerClasses(self):
        self.content = CalcInputContent(self)
        self.grNode = CalcGraphicsNode(self)
        self.grNode.height=300
        self.grNode.width=200
        self.content.h_edit.textChanged.connect(self.onInputChanged)
        self.content.P_edit.textChanged.connect(self.onInputChanged)
        self.content.F_kgs_edit.textChanged.connect(self.onInputChanged)
        self.content.fluid.currentIndexChanged.connect(self.onInputChanged)

    def evalImplementation(self):
        u_h = self.content.h_edit.text()
        s_h = float(u_h)
        
        u_P = self.content.P_edit.text()
        s_P = float(u_P)
        
        u_F_kgs = self.content.F_kgs_edit.text()
        s_F_kgs = float(u_F_kgs)
        
        u_fluid = self.content.fluid.currentText()
        s_fluid = u_fluid
        
        
        
        
        fluid=self.content.fluid.currentText()
        
        #self.h=PropsSI("H", "P", 100000*s_P, "H", s_h*1000, fluid)/1000
        self.h=s_h
        
        self.value = [s_fluid,s_F_kgs,s_P,self.h]
        
        self.markDirty(False)
        self.markInvalid(False)

        self.markDescendantsInvalid(False)
        self.markDescendantsDirty()

        self.grNode.setToolTip("")

        self.evalChildren()
      #  print("données entrée = ",self.value)
        return self.value
   
class Fluid:
    def __init__(self):
        self.fluid_list=["1-Butene","Acetone"	,"Air"	,"Ammonia","Argon"	,"Benzene"	,"CarbonDioxide"	,"CarbonMonoxide","CarbonylSulfide"	,
"CycloHexane"	,"CycloPropane"	,"Cyclopentane"	,"D4"	,"D5"	,"D6"	,"Deuterium"	,"Dichloroethane"	,"DiethylEther"	,
"DimethylCarbonate"	,
"DimethylEther"	,
"Ethane"	,
"Ethanol"	,
"EthylBenzene"	,
"Ethylene"	,
"EthyleneOxide"	,
"Fluorine"	,
"HFE143m"	,
"HeavyWater"	,
"Helium"	,
"Hydrogen"	,
"HydrogenChloride"	,
"HydrogenSulfide"	,
"IsoButane"	,
"IsoButene"	,
"Isohexane"	,
"Isopentane"	,
"Krypton"	,
"MD2M"	,
"MD3M"	,
"MD4M"	,
"MDM"	,
"MM"	,
"Methane"	,
"Methanol"	,
"MethylLinoleate"	,
"MethylLinolenate"	,
"MethylOleate"	,
"MethylPalmitate"	,
"MethylStearate"	,
"Neon"	,
"Neopentane"	,
"Nitrogen"	,
"NitrousOxide"	,
"Novec649"	,
"OrthoDeuterium"	,
"OrthoHydrogen"	,
"Oxygen"	,
"ParaDeuterium"	,
"ParaHydrogen"	,
"Propylene"	,
"Propyne"	,
"R11"	,
"R113"	,
"R114"	,
"R115"	,
"R116"	,
"R12"	,
"R123"	,
"R1233zd(E)"	,
"R1234yf"	,
"R1234ze(E)"	,
"R1234ze(Z)"	,
"R124"	,
"R125"	,
"R13"	,
"R134a"	,
"R13I1"	,
"R14"	,
"R141b"	,
"R142b"	,
"R143a"	,
"R152A"	,
"R161"	,
"R21"	,
"R218"	,
"R22"	,
"R227EA"	,
"R23"	,
"R236EA"	,
"R236FA"	,
"R245ca"	,
"R245fa"	,
"R32"	,
"R365MFC"	,
"R40"	,
"R404A"	,
"R407C"	,
"R41"	,
"R410A"	,
"R507A"	,
"RC318"	,
"SES36"	,
"SulfurDioxide"	,
"SulfurHexafluoride"	,
"Toluene"	,
"Water"	,
"Xenon"	,
"cis-2-Butene"	,
"m-Xylene"	,
"n-Butane"	,
"n-Decane"	,
"n-Dodecane"	,
"n-Heptane"	,
"n-Hexane"	,
"n-Nonane"	,
"n-Octane"	,
"n-Pentane"	,
"n-Propane"	,
"n-Undecane"	,
"o-Xylene"	,
"p-Xylene"	,
"trans-2-Butene"	]
    
    def listeDesFluides(self):
        return self.fluid_list
       
            


class FluidPort:
    def __init__(self):
        self.F=0
        self.P = 101325
        self.h = 10000
        self.fluid = "ammonia"
        
    def propriete(self,Pro,I1,ValI1,I2,ValI2):
        result=PropsSI(Pro, I1, ValI1, I2, ValI2, self.fluid)
        return result


from ThermodynamicCycles.FluidPort.FluidPort import FluidPort
from CoolProp.CoolProp import PropsSI
import pandas as pd 

class Object :
    def __init__(self):
        self.Inlet=FluidPort() 
        self.Outlet=FluidPort()
        self.Tl_sat=0
        self.Hl_sat = 0
        self.Sl_sat = 0
        self.subcooling=2
        self.Q_cond=0
        #Output Data
        self.df=[]
        
    def calculate (self):
        self.Tl_sat=PropsSI('T','P',self.Inlet.P,'Q',0,self.Inlet.fluid)
       
        self.Hl_sat =PropsSI('H','P',self.Inlet.P,'Q',0,self.Inlet.fluid)
        #print("H5=",self.Hl_sat )
        #print("self.Inlet.P=",self.Inlet.P )
        #print("self.Inlet.fluid=",self.Inlet.fluid )
        
        self.Sl_sat =PropsSI('S','P',self.Inlet.P,'Q',0,self.Inlet.fluid)
        
        self.To=self.Tl_sat-self.subcooling
        self.Ho = PropsSI('H','P',self.Inlet.P,'T',self.To,self.Inlet.fluid)
        self.So = PropsSI('S','P',self.Inlet.P,'T',self.To,self.Inlet.fluid)
        self.Outlet.fluid=self.Inlet.fluid
        self.Outlet.h=self.Ho
        self.Outlet.F=self.Inlet.F
        self.Outlet.P=self.Inlet.P
        
        self.Q_cond=self.Inlet.F*(self.Inlet.h-self.Outlet.h)

        #self.Outlet.calculate_properties()
        self.df = pd.DataFrame({'Condenser': [self.Inlet.fluid,self.Outlet.F,self.Tl_sat-273.15,self.Hl_sat/1000,self.Sl_sat/1000,self.To-273.15,self.Ho/1000,self.So/1000,self.Q_cond/1000,], },
                      index = ['fluid','Outlet.F',"Tl_sat(°C)",    "Hl_sat(kJ/kg)",      "Sl_sat(kJ/kg-K)",       "To(°C)",      "Ho(kJ/kg)",       "So(kJ/kg-K)",     "Q_cond(kW)", ])

      


import os
import sys
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *

#sys.path.insert(0, os.path.join( os.path.dirname(__file__), "..", ".." ))


from NodeEditor.nodeeditor.utils import loadStylesheets
from NodeEditor.nodeeditor.node_editor_window import NodeEditorWindow
from PyqtSimulator.calc_sub_window import CalculatorSubWindow
from PyqtSimulator.calc_drag_listbox import QDMDragListbox
from NodeEditor.nodeeditor.utils import dumpException, pp
from PyqtSimulator.calc_conf import *

# Enabling edge validators
from NodeEditor.nodeeditor.node_edge import Edge
from NodeEditor.nodeeditor.node_edge_validators import *
Edge.registerEdgeValidator(edge_validator_debug)
Edge.registerEdgeValidator(edge_cannot_connect_two_outputs_or_two_inputs)
Edge.registerEdgeValidator(edge_cannot_connect_input_and_output_of_same_node)


# images for the dark skin
import PyqtSimulator.qss.nodeeditor_dark_resources


DEBUG = False


class CalculatorWindow(NodeEditorWindow):

    def initUI(self):
        self.name_company = 'Blenderfreak'
        self.name_product = 'Calculator NodeEditor'

        self.stylesheet_filename = os.path.join(os.path.dirname(__file__), "qss/nodeeditor.qss")
        loadStylesheets(
            os.path.join(os.path.dirname(__file__), "qss/nodeeditor-dark.qss"),
            self.stylesheet_filename
        )

        self.empty_icon = QIcon(".")

        if DEBUG:
            print("Registered nodes:")
            pp(CALC_NODES)


        self.mdiArea = QMdiArea()
        self.mdiArea.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.mdiArea.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.mdiArea.setViewMode(QMdiArea.TabbedView)
        self.mdiArea.setDocumentMode(True)
        self.mdiArea.setTabsClosable(True)
        self.mdiArea.setTabsMovable(True)
        self.setCentralWidget(self.mdiArea)

        self.mdiArea.subWindowActivated.connect(self.updateMenus)
        self.windowMapper = QSignalMapper(self)
        self.windowMapper.mapped[QWidget].connect(self.setActiveSubWindow)

        self.createNodesDock()

        self.createActions()
        self.createMenus()
        self.createToolBars()
        self.createStatusBar()
        self.updateMenus()

        self.readSettings()

        self.setWindowTitle("Calcul des systèmes énergétiques")

    def closeEvent(self, event):
        self.mdiArea.closeAllSubWindows()
        if self.mdiArea.currentSubWindow():
            event.ignore()
        else:
            self.writeSettings()
            event.accept()
            # hacky fix for PyQt 5.14.x
            import sys
            sys.exit(0)


    def createActions(self):
        super().createActions()

        self.actClose = QAction("Cl&ose", self, statusTip="Close the active window", triggered=self.mdiArea.closeActiveSubWindow)
        self.actCloseAll = QAction("Close &All", self, statusTip="Close all the windows", triggered=self.mdiArea.closeAllSubWindows)
        self.actTile = QAction("&Tile", self, statusTip="Tile the windows", triggered=self.mdiArea.tileSubWindows)
        self.actCascade = QAction("&Cascade", self, statusTip="Cascade the windows", triggered=self.mdiArea.cascadeSubWindows)
        self.actNext = QAction("Ne&xt", self, shortcut=QKeySequence.NextChild, statusTip="Move the focus to the next window", triggered=self.mdiArea.activateNextSubWindow)
        self.actPrevious = QAction("Pre&vious", self, shortcut=QKeySequence.PreviousChild, statusTip="Move the focus to the previous window", triggered=self.mdiArea.activatePreviousSubWindow)

        self.actSeparator = QAction(self)
        self.actSeparator.setSeparator(True)

        self.actAbout = QAction("&About", self, statusTip="Show the application's About box", triggered=self.about)

    def getCurrentNodeEditorWidget(self):
        """ we're returning NodeEditorWidget here... """
        activeSubWindow = self.mdiArea.activeSubWindow()
        if activeSubWindow:
            return activeSubWindow.widget()
        return None

    def onFileNew(self):
        try:
            subwnd = self.createMdiChild()
            subwnd.widget().fileNew()
            subwnd.show()
        except Exception as e: dumpException(e)


    def onFileOpen(self):
        fnames, filter = QFileDialog.getOpenFileNames(self, 'Open graph from file', self.getFileDialogDirectory(), self.getFileDialogFilter())

        try:
            for fname in fnames:
                if fname:
                    existing = self.findMdiChild(fname)
                    if existing:
                        self.mdiArea.setActiveSubWindow(existing)
                    else:
                        # we need to create new subWindow and open the file
                        nodeeditor = CalculatorSubWindow()
                        if nodeeditor.fileLoad(fname):
                            self.statusBar().showMessage("File %s loaded" % fname, 5000)
                            nodeeditor.setTitle()
                            subwnd = self.createMdiChild(nodeeditor)
                            subwnd.show()
                        else:
                            nodeeditor.close()
        except Exception as e: dumpException(e)


    def about(self):
        QMessageBox.about(self, "About Calculator NodeEditor Example",
                "The <b>Calculator NodeEditor</b> example demonstrates how to write multiple "
                "document interface applications using PyQt5 and NodeEditor. For more information visit: "
                "<a href='https://www.blenderfreak.com/'>www.BlenderFreak.com</a>")

    def createMenus(self):
        super().createMenus()

        self.windowMenu = self.menuBar().addMenu("&Window")
        self.updateWindowMenu()
        self.windowMenu.aboutToShow.connect(self.updateWindowMenu)

        self.menuBar().addSeparator()

        self.helpMenu = self.menuBar().addMenu("&Help")
        self.helpMenu.addAction(self.actAbout)

        self.editMenu.aboutToShow.connect(self.updateEditMenu)

    def updateMenus(self):
        # print("update Menus")
        active = self.getCurrentNodeEditorWidget()
        hasMdiChild = (active is not None)

        self.actSave.setEnabled(hasMdiChild)
        self.actSaveAs.setEnabled(hasMdiChild)
        self.actClose.setEnabled(hasMdiChild)
        self.actCloseAll.setEnabled(hasMdiChild)
        self.actTile.setEnabled(hasMdiChild)
        self.actCascade.setEnabled(hasMdiChild)
        self.actNext.setEnabled(hasMdiChild)
        self.actPrevious.setEnabled(hasMdiChild)
        self.actSeparator.setVisible(hasMdiChild)

        self.updateEditMenu()

    def updateEditMenu(self):
        try:
            # print("update Edit Menu")
            active = self.getCurrentNodeEditorWidget()
            hasMdiChild = (active is not None)

            self.actPaste.setEnabled(hasMdiChild)

            self.actCut.setEnabled(hasMdiChild and active.hasSelectedItems())
            self.actCopy.setEnabled(hasMdiChild and active.hasSelectedItems())
            self.actDelete.setEnabled(hasMdiChild and active.hasSelectedItems())

            self.actUndo.setEnabled(hasMdiChild and active.canUndo())
            self.actRedo.setEnabled(hasMdiChild and active.canRedo())
        except Exception as e: dumpException(e)



    def updateWindowMenu(self):
        self.windowMenu.clear()

        toolbar_nodes = self.windowMenu.addAction("Nodes Toolbar")
        toolbar_nodes.setCheckable(True)
        toolbar_nodes.triggered.connect(self.onWindowNodesToolbar)
        toolbar_nodes.setChecked(self.nodesDock.isVisible())

        self.windowMenu.addSeparator()

        self.windowMenu.addAction(self.actClose)
        self.windowMenu.addAction(self.actCloseAll)
        self.windowMenu.addSeparator()
        self.windowMenu.addAction(self.actTile)
        self.windowMenu.addAction(self.actCascade)
        self.windowMenu.addSeparator()
        self.windowMenu.addAction(self.actNext)
        self.windowMenu.addAction(self.actPrevious)
        self.windowMenu.addAction(self.actSeparator)

        windows = self.mdiArea.subWindowList()
        self.actSeparator.setVisible(len(windows) != 0)

        for i, window in enumerate(windows):
            child = window.widget()

            text = "%d %s" % (i + 1, child.getUserFriendlyFilename())
            if i < 9:
                text = '&' + text

            action = self.windowMenu.addAction(text)
            action.setCheckable(True)
            action.setChecked(child is self.getCurrentNodeEditorWidget())
            action.triggered.connect(self.windowMapper.map)
            self.windowMapper.setMapping(action, window)

    def onWindowNodesToolbar(self):
        if self.nodesDock.isVisible():
            self.nodesDock.hide()
        else:
            self.nodesDock.show()

    def createToolBars(self):
        pass

    def createNodesDock(self):
        self.nodesListWidget = QDMDragListbox()

        self.nodesDock = QDockWidget("Nodes")
        self.nodesDock.setWidget(self.nodesListWidget)
        self.nodesDock.setFloating(False)

        self.addDockWidget(Qt.RightDockWidgetArea, self.nodesDock)

    def createStatusBar(self):
        self.statusBar().showMessage("Ready")

    def createMdiChild(self, child_widget=None):
        nodeeditor = child_widget if child_widget is not None else CalculatorSubWindow()
        subwnd = self.mdiArea.addSubWindow(nodeeditor)
        subwnd.setWindowIcon(self.empty_icon)
        # nodeeditor.scene.addItemSelectedListener(self.updateEditMenu)
        # nodeeditor.scene.addItemsDeselectedListener(self.updateEditMenu)
        nodeeditor.scene.history.addHistoryModifiedListener(self.updateEditMenu)
        nodeeditor.addCloseEventListener(self.onSubWndClose)
        return subwnd

    def onSubWndClose(self, widget, event):
        existing = self.findMdiChild(widget.filename)
        self.mdiArea.setActiveSubWindow(existing)

        if self.maybeSave():
            event.accept()
        else:
            event.ignore()


    def findMdiChild(self, filename):
        for window in self.mdiArea.subWindowList():
            if window.widget().filename == filename:
                return window
        return None


    def setActiveSubWindow(self, window):
        if window:
            self.mdiArea.setActiveSubWindow(window)


from Facture.ATR_Transport_Distribution import input_Contrat, input_Facture, input_Tarif, ATR_calculation

if __name__ == "__main__":
    contrat = input_Contrat(type_tarif_acheminement='T4',CJN_MWh_j=93,modulation_MWh_j=20.217, CAR_MWh=6801.540, profil="P019", station_meteo="PARIS-MONTSOURIS", reseau_transport="GRTgaz", niv_tarif_region=2)
    facture = input_Facture(start="2024-06-01", end="2024-06-30", kWh_total=0)
    tarif = input_Tarif(prix_kWh=0.03171+0.00571)

    atr = ATR_calculation(contrat, facture, tarif)
    atr.calculate()
    print(atr.df)
    print(atr.df_transport)
    print(atr.df_distribution)
    print(atr.df_taxes_contributions)
    print(atr.df_molecule)
    print(atr.df_annuel)

    print("atr.CJN_MWh_j",atr.CJN)
    #zi
    print("art.cofficient_zi", atr.zi)
    #coef A
    print("atr.cofficient_A", atr.coef_A)

    print("coef_stockage", atr.coef_stockage)

   


from ThermodynamicCycles.FluidPort.FluidPort import FluidPort


from CoolProp.CoolProp import PropsSI

class Object:
    def __init__(self):
        
        ####parameter
        self.eta_is=0.7
        self.MecEff = 1 #"rendement mecanique" 
        self.VolEff = 0.99 # "rendement volumetrique";
        self.cyl= 0.02 #cylindrée en m3
        self.f=50 #SIunits.Frequency rotation
        self.p = 1 #"Nb pole/2" 
        #rendement volumetrique
        self.a0=0.95 #piston, à vis= 0.9
        self.a1=0.038 #piston, à vis= 0.008
        #rendement isentropique
        self.K1=0.8
        self.K2=0.0037
        self.K3=-0.16
        self.R1=7
        self.R2=1.2
        
        
        self.Inlet=FluidPort() 
        self.F=0.1
       # self.Inlet.F=self.F
        self.Outlet=FluidPort()
        self.So_is=0
        self.To_is=0
        self.Ho_is=0
        self.HP=15*100000
        self.Ho_ref=0
        self.To_ref=0
        self.So_ref=0
        self.Tdischarge_target=80
        self.To=0
        
        
        self.Pu = 50000 # "Puissance utile";
        self.Q_losses=0
        
        #######################################################Dymola
        self.Pel=1000 #Puissance electrique W
        self.Pth= 50000 ## Puissance mécanique theorique isentropique
        self.VitesseDeRotation=0 # "tr/min";
        
        self.Inlet_rho=0 #masse volumique à l'aspiration kg/m3
        self.Vol_Bal=0 #m3/s
        self.Vol_asp=0 #m3/s
        self.Taux=1 #taux de compression
        
        
        
    def calculate (self):
        
        self.Taux=self.HP/self.Inlet.P
        self.eta_is=self.K1+self.K2*(self.Taux-self.R1)**2+self.K3/(self.Taux-self.R2)
        print("self.eta_is=",self.eta_is)
        self.VolEff =self.a0-self.a1*self.Taux
        print("self.VolEff=",self.VolEff)
        self.Inlet_rho = PropsSI('D','P',self.Inlet.P,'H',self.Inlet.h,self.Inlet.fluid)
        print("self.Inlet_rho=",self.Inlet_rho)
        
        self.VitesseDeRotation=(self.f/self.p)*60
        print("self.VitesseDeRotation=",self.VitesseDeRotation)
        
        self.Vol_Bal = self.cyl * (self.VitesseDeRotation / 60)
        print("self.Vol_Bal=",self.Vol_Bal)
        self.F = self.VolEff * self.Vol_Bal * self.Inlet_rho
        print("self.F=",self.F)
        
        self.Vol_asp = self.F / self.Inlet_rho
        print("self.Vol_asp=",self.Vol_asp)
      
      
        #self.F=self.Inlet.F #débit imposé par le compresseur
        self.So_is = PropsSI('S','P',self.Inlet.P,'H',self.Inlet.h,self.Inlet.fluid)
        self.To_is=PropsSI('T','P',self.HP,'S',self.So_is,self.Inlet.fluid)
        self.Ho_is = PropsSI('H','P',self.HP,'S',self.So_is,self.Inlet.fluid)
        
        self.Ho_ref = (self.Ho_is-self.Inlet.h)/self.eta_is+self.Inlet.h
        self.To_ref=PropsSI('T','P',self.HP,'H',self.Ho_ref,self.Inlet.fluid)
        self.So_ref=PropsSI('S','P',self.HP,'H',self.Ho_ref,self.Inlet.fluid)
        
        self.To=self.Tdischarge_target+273.15
        self.Ho=PropsSI('H','P',self.HP,'T',self.To,self.Inlet.fluid)
        self.So=PropsSI('S','P',self.HP,'T',self.To,self.Inlet.fluid)
        
        self.Outlet.fluid=self.Inlet.fluid
        self.Outlet.h=self.Ho
        self.Outlet.F=self.F
        self.Outlet.P=self.HP
        
        self.Pu=self.F*(self.Ho_ref-self.Inlet.h)
        print("self.Pu=",self.Pu)
        
        self.Q_losses=self.F*(self.Ho_ref-self.Ho)
        print("self.Q_losses=",self.Q_losses)
       # print("To_is=",self.To_is-273.15,"H3is=",self.Ho_is,"T3ref=",self.To_ref-273.15,"H3ref=",self.Ho_ref,"To=",self.To-273.15,"Ho=",self.Ho)
       # print("Q_comp=",self.Pu)
       # print("Q_losses=",self.Q_losses)
       # print("self.Inlet.F=",self.Inlet.F)
       
        self.Pth = self.F * (self.Ho_is-self.Inlet.h)
        self.Pel = self.Pth / (self.eta_is * self.MecEff)
        print("self.Pth=",self.Pth)
        print("self.Pel=",self.Pel)
        
        
  
     

     
    

      
     
     

     

     
      
     
      
        
        
    
     
        


# -*- encoding: utf-8 -*-
"""
Module with some helper functions
"""
import traceback
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from pprint import PrettyPrinter

pp = PrettyPrinter(indent=4).pprint


def dumpException(e=None):
    """
    Prints out Exception message with traceback to the console

    :param e: Exception to print out
    :type e: Exception
    """
    # print("%s EXCEPTION:" % e.__class__.__name__, e)
    # traceback.print_tb(e.__traceback__)
    traceback.print_exc()


def loadStylesheet(filename:str):
    """
    Loads an qss stylesheet to current QApplication instance

    :param filename: Filename of qss stylesheet
    :type filename: str
    """
    print('STYLE loading:', filename)
    file = QFile(filename)
    file.open(QFile.ReadOnly | QFile.Text)
    stylesheet = file.readAll()
    QApplication.instance().setStyleSheet(str(stylesheet, encoding='utf-8'))

def loadStylesheets(*args):
    """
    Loads multiple qss stylesheets. Concats them together and applies the final stylesheet to current QApplication instance

    :param args: variable number of filenames of qss stylesheets
    :type args: str, str,...
    """
    res = ''
    for arg in args:
        file = QFile(arg)
        file.open(QFile.ReadOnly | QFile.Text)
        stylesheet = file.readAll()
        res += "\n" + str(stylesheet, encoding='utf-8')
    QApplication.instance().setStyleSheet(res)



from AHU.air_humide import air_humide



class AirPort:
    def __init__(self):
        self.F=None # air humide en kg/s
        self.F_dry=None # air sec en kg/s
        self.P = 101325 # pression
        self.h = None # enthalpie spéc
        self.w = None # Humidité absolue
        
    # def propriete(self):
    #     self.result="RH,Pv_sat,T="
    #     self.T=air_humide.Air_T_db(h=self.h, w=self.w)
    #     self.Pv_sat=air_humide.Air_Pv_sat(self.T)
    #     self.RH=air_humide.Air_RH(Pv_sat=self.Pv_sat, w=self.w, P=self.P)
    #     return self.result,self.RH,self.Pv_sat,self.T

        # Propriétés calculées
        self._T = None       # Température de l'air en °C (calculée si nécessaire)
        self._RH = None      # Humidité relative de l'air en % (calculée si nécessaire)
        self._Pv_sat = None  # Pression de vapeur saturante (Pa) (calculée si nécessaire)

    @property
    def T(self):
        """
        Calcul automatique de la température de l'air en fonction de l'enthalpie et de l'humidité absolue.
        """
        if self._T is None and self.h is not None and self.w is not None:
            self._T = air_humide.Air_T_db(h=self.h, w=self.w)  # Calcul de la température en 
        return self._T

    @property
    def RH(self):
        """
        Calcul automatique de l'humidité relative en fonction de la température, de l'humidité absolue et de la pression.
        """
        if self._RH is None and self.w is not None and self.P is not None:
            # Calcul de la pression de vapeur saturante et de l'humidité relative
            self._Pv_sat = air_humide.Air_Pv_sat(self.T)  # Calcul de la pression de vapeur saturante
            self._RH = air_humide.Air_RH(Pv_sat=self._Pv_sat, w=self.w, P=self.P)  # Calcul de RH
        return self._RH

    @property
    def Pv_sat(self):
        """
        Calcul automatique de la pression de vapeur saturante en fonction de la température.
        """
        if self._Pv_sat is None and self.T is not None:
            self._Pv_sat = air_humide.Air_Pv_sat(self.T)  # Calcul de la pression de vapeur saturante
        return self._Pv_sat

  
    def update_properties(self):
        """
        Méthode qui permet de forcer le recalcul des propriétés (T, RH, Pv_sat) si nécessaire.
        Cela permet de mettre à jour les propriétés automatiquement si les valeurs de base changent.
        """
        self._T = None
        self._RH = None
        self._Pv_sat = None
        # Recalcule les propriétés dès qu'elles sont nécessaires
        _ = self.T
        _ = self.RH
        _ = self.Pv_sat  




from PyQt5.QtCore import *
from PyqtSimulator.calc_conf import *
from PyqtSimulator.calc_node_base import *
from NodeEditor.nodeeditor.utils import dumpException

#############les modèles d'un groupe frigorifique#################
#from Modules_GroupeFrigorifique.Evaporator import Evaporator
from ThermodynamicCycles.Compressor import Compressor
#from Modules_GroupeFrigorifique.Desuperheater import Desuperheater
#from Modules_GroupeFrigorifique.Expansion_Valve import Expansion_Valve
#from Modules_GroupeFrigorifique.Condenser import Condenser
from ThermodynamicCycles.Connect import Fluid_connect
from ThermodynamicCycles.FluidPort.FluidPort import FluidPort


@register_node(OP_NODE_MIX)
class CalcNode_Mix(CalcNode):
    icon = "icons/mixer.png"
    op_code = OP_NODE_MIX
    op_title = "Mélangeur"
    content_label = "+"
    content_label_objname = "calc_node_bg"
    
 

    def evalOperation(self, input1, input2):
        self.value=[]
        
       
        self.value.append(input1[0]) #fluide
        self.value.append(input1[1]+input2[1]) #débit
        self.value.append(min(input1[2],input2[2])) #pression min
        self.value.append((input1[1]*input1[3]+input2[1]*input2[3])/(input1[1]+input2[1])) #Enthalpie
            
        return self.value
    
@register_node(OP_NODE_ADD)
class CalcNode_Add(CalcNode):
    icon = "icons/add.png"
    op_code = OP_NODE_ADD
    op_title = "Add"
    content_label = "+"
    content_label_objname = "calc_node_bg"

    def evalOperation(self, input1, input2):
        self.value=[]
        for i in range(min(len(input1),len(input2))):
            self.value.append(input1[i]+input2[i])
       # print("input1 = ",input1)
      #  print("input2 = ",input2)
      #  print("somme = ",self.value)
        return self.value
    
    

@register_node(OP_NODE_SUB)
class CalcNode_Sub(CalcNode):
    icon = "icons/sub.png"
    op_code = OP_NODE_SUB
    op_title = "Substract"
    content_label = "-"
    content_label_objname = "calc_node_bg"

    def evalOperation(self, input1, input2):
        self.value=[]
        
       
        self.value.append(input1[0]) #fluide
        self.value.append(input1[1]-input2[1]) #diff_débit
        self.value.append((input1[2]+input2[2])/2) #moy Pression
        self.value.append(input1[3]-input2[3]) #d_Enthalpie
            
        return self.value

@register_node(OP_NODE_MUL)
class CalcNode_Mul(CalcNode):
    icon = "icons/mul.png"
    op_code = OP_NODE_MUL
    op_title = "Multiply"
    content_label = "*"
    content_label_objname = "calc_node_mul"

    def evalOperation(self, input1, input2):
        print('foo')
        return input1 * input2

@register_node(OP_NODE_DIV)
class CalcNode_Div(CalcNode):
    icon = "icons/divide.png"
    op_code = OP_NODE_DIV
    op_title = "Divide"
    content_label = "/"
    content_label_objname = "calc_node_div"

    def evalOperation(self, input1, input2):
        return input1 / input2

# way how to register by function call
# register_node_now(OP_NODE_ADD, CalcNode_Add)


def DJU_costic(Tmin, Tmax, base_chauffage=18, base_refroidissement=23):
    dju_chauffage = 0
    dju_rafraichissement = 0
    
    if Tmax <= base_chauffage:
        dju_chauffage = base_chauffage - (Tmin + Tmax) / 2
        
    elif Tmin >= base_refroidissement:
        dju_rafraichissement = (Tmin + Tmax) / 2 - base_refroidissement
        
    else:
        if Tmin < base_chauffage and Tmax > base_chauffage:
            a = Tmax - Tmin
            b = (base_chauffage - Tmin) / a
            dju_chauffage = a * b * (0.08 + 0.42 * b)
        
        if Tmax > base_refroidissement and Tmin < base_refroidissement:
            a = Tmax - base_refroidissement
            b = a / (Tmax - Tmin)
            dju_rafraichissement = a * b * (0.08 + 0.42 * b)
        
    return dju_chauffage, dju_rafraichissement



"""
Reverse-engineering of Delphi enthalpy model for the deethanizer (Chapter 6).

CONDENSEUR PARTIEL vs TOTAL
============================
Le modele Python (eq. 5.54 de Deethanizer.py) utilise un CONDENSEUR PARTIEL:
    HD = H[0]   (enthalpy vapeur au condenseur)
    QC = V[1]*H[1] - L[0]*h[0] - D*HD
       = V[1]*(H[1]-H[0]) + L[0]*(H[0]-h[0])

Terme A: V[1]*(H[1]-H[0]) = refroidissement sensible de tout le vapeur
Terme B: L[0]*(H[0]-h[0]) = condensation du reflux L[0] seulement
         (D quitte en VAPEUR, donc n apporte pas de chaleur latente)

BILAN GLOBAL:
    QR = -hF_total + B*h[B] + D*H[0] + QC

PHI (flux d enthalpie montant net dans chaque section adiabatique):
    phi = V[j+1]*H[j+1] - L[j]*h[j]  = const

Pour le condenseur partiel, le bilan cumule depuis le haut:
    D*H[0] + QC = V[1]*H[1] - L[0]*h[0]    [definition QC]
    => phi_rectif = V[1]*H[1] - L[0]*h[0] = D*H[0] + QC

STRATEGIE D ANALYSE:
    1. Decomposer QC en termes A + B pour Python et Delphi
    2. Terme A (refroidissement vapeur): compare H[1]-H[0] Python vs Delphi
    3. Terme B (condensation): compare H[0]-h[0] Python vs Delphi
    4. V[1], L[0], D sont EXACTEMENT les memes => erreur QC = erreur enthalpie pure
    5. Identifier si c est ΔHvap_Watson ou H_PR_departure qui cause l ecart
"""

import sys
import numpy as np

sys.path.insert(0, 'src')

from Distillation.Deethanizer import Deethanizer

# =============================================================================
# PART 1 - Reference data Delphi (Chapitre 6, Tables 6-7)
# =============================================================================

QC_REF = 4_133_587.91   # kJ/h
QR_REF = 22_854_385.49  # kJ/h

T_REF = np.array([
    260.2, 283.6, 293.5, 299.1, 302.9, 320.8, 337.6, 343.1, 345.0, 346.1,
    346.5, 346.7, 346.8, 346.8, 346.9, 346.9, 347.0, 347.0, 347.2, 347.4,
    348.0, 352.3, 353.4, 354.3, 355.6, 357.5, 360.9, 367.1, 380.2, 416.6
])
V_REF = np.array([
    562.1, 868.3, 900.2738, 923.6122, 925.198, 818.9466, 1076.698, 1676.743,
    2072.401, 2544.798, 2485.902, 2429.856, 2400.084, 2377.905, 2368.245,
    2350.846, 2336.727, 2313.368, 2297.692, 2277.709, 2239.937, 1995.84,
    2384.162, 2430.911, 2357.319, 2229.362, 2065.881, 1862.577, 1592.462,
    1156.98
])
L_REF = np.array([
    306.2, 338.1738, 361.5122, 363.098, 428.6541, 1438.504, 2038.548,
    2434.207, 2906.603, 2847.708, 2791.662, 2761.889, 2739.711, 2730.051,
    2712.652, 2698.533, 2675.173, 2659.498, 2639.514, 2601.743, 2395.55,
    3108.562, 3155.311, 3081.719, 2953.762, 2790.281, 2586.977, 2316.862,
    1881.38, 724.4
])

N     = 30
D_REF = V_REF[0]   # 562.1 kmol/h  (distillat vapeur V[0]=D pour condenseur partiel)
B_REF = L_REF[29]  # 724.4 kmol/h

VFC_total = 171.81
LFC_total = 752.10
VFH_total = 37.90
LFH_total = 324.73
FC_T      = 255.15  # K
FH_T      = 388.15  # K
P_BAR     = 27.49   # bar

# =============================================================================
# PART 2 - Modele Python, Cas 1
# =============================================================================
print("\n" + "=" * 65)
print("PYTHON MODEL - Case 1 convergence")
print("=" * 65)

col = Deethanizer()
col.solve(case=1, max_outer=20, tol=1.0, verbose=True)

T_PY = col.T.copy()
V_PY = col.V.copy()
L_PY = col.L.copy()
QC_PY = abs(col.QC)
QR_PY = abs(col.QR)
P_Pa  = col.P  # Pa (stored in Pa internally)
P_bar = P_Pa / 1e5

# Fractions molaires converges
l_conv = col.l    # shape (nc, N)
v_conv = col.v
x_PY   = np.zeros_like(l_conv)
y_PY   = np.zeros_like(v_conv)
for j in range(N):
    Lj = l_conv[:, j].sum()
    Vj = v_conv[:, j].sum()
    if Lj > 1e-10:
        x_PY[:, j] = l_conv[:, j] / Lj
    if Vj > 1e-10:
        y_PY[:, j] = v_conv[:, j] / Vj

# =============================================================================
# PART 3 - Enthalpies Python par etage (etat converge)
# =============================================================================
h_PY = np.zeros(N)   # kJ/kmol, liquide
H_PY = np.zeros(N)   # kJ/kmol, vapeur

for j in range(N):
    h_PY[j] = col.eos.pure_component_enthalpy(T_PY[j], P_Pa, x_PY[:, j], "liquid")
    H_PY[j] = col.eos.mixture_enthalpy(T_PY[j], P_Pa, y_PY[:, j], "vapor")

# Verification: QC_PY doit = V[1]*(H[1]-H[0]) + L[0]*(H[0]-h[0])
# Pour condenseur PARTIEL:  D quitte en vapeur => HD = H[0]
A_term_py = V_PY[1] * (H_PY[1] - H_PY[0])   # refroidissement sensible
B_term_py = L_PY[0] * (H_PY[0] - h_PY[0])   # condensation reflux
QC_check   = A_term_py + B_term_py

print(f"\n  Verification formule condenseur partiel:")
print(f"    Terme A = V[1]*(H[1]-H[0]) = {A_term_py:>12,.1f} kJ/h  ({A_term_py/QC_PY*100:.1f}%)")
print(f"    Terme B = L[0]*(H[0]-h[0]) = {B_term_py:>12,.1f} kJ/h  ({B_term_py/QC_PY*100:.1f}%)")
print(f"    QC_check = A+B             = {QC_check:>12,.1f} kJ/h")
print(f"    QC_py (solver)             = {QC_PY:>12,.1f} kJ/h")
print(f"    Ecart: {QC_check - QC_PY:+.1f} kJ/h  (doit etre ~0)")

print(f"\n  H[1]-H[0] = {H_PY[1]-H_PY[0]:>8.2f} kJ/kmol  (refroidissement vapeur {T_PY[1]:.1f}K -> {T_PY[0]:.1f}K)")
print(f"  H[0]-h[0] = {H_PY[0]-h_PY[0]:>8.2f} kJ/kmol  (chaleur latente de condensation a T[0]={T_PY[0]:.1f}K)")

# =============================================================================
# PART 4 - Decomposition de l erreur QC: Terme A vs Terme B
# =============================================================================
print("\n" + "=" * 70)
print("DECOMPOSITION ERREUR QC: refroidissement sensible vs condensation")
print("=" * 70)

D_val  = D_REF   # = V_REF[0] = 562.1 kmol/h
L0_val = L_REF[0]  # = 306.2 kmol/h
V1_val = V_REF[1]  # = 868.3 kmol/h

# Terme A Python (refroidissement sensible vapeur dV1 de T[1] a T[0])
A_py = V_PY[1] * (H_PY[1] - H_PY[0])
B_py = L_PY[0] * (H_PY[0] - h_PY[0])

print(f"\n  PYTHON (condenseur partiel, V[1]={V_PY[1]:.1f}, L[0]={L_PY[0]:.1f}):")
print(f"    H[1]-H[0] = {H_PY[1]-H_PY[0]:>8.2f} kJ/kmol")
print(f"    H[0]-h[0] = {H_PY[0]-h_PY[0]:>8.2f} kJ/kmol  [ΔHvap eff. condenseur]")
print(f"    Terme A   = {A_py:>12,.1f} kJ/h ({A_py/QC_PY*100:.1f}%)")
print(f"    Terme B   = {B_py:>12,.1f} kJ/h ({B_py/QC_PY*100:.1f}%)")
print(f"    QC_py     = {QC_PY:>12,.1f} kJ/h")

# Terme A et B impliques pour Delphi
# Hypothese 1: seul le terme B differe (H[1]-H[0] identique entre Python et Delphi)
# => B_delphi = QC_ref - A_py
B_delphi_h1  = QC_REF - A_py
HH_delphi_h1 = B_delphi_h1 / L0_val

# Hypothese 2: seul le terme A differe (H[0]-h[0] identique)
A_delphi_h2 = QC_REF - B_py
HH_delphi_h2 = A_delphi_h2 / V1_val

# Hypothese 3: erreur proportionnelle (meme ratio sur A et B)
scale_3 = QC_REF / QC_PY

print(f"\n  DELPHI IMPLIQUE (V[1]={V1_val:.1f}, L[0]={L0_val:.1f}, QC_ref={QC_REF:,.1f}):")
print(f"    QC_ref                              = {QC_REF:>12,.1f} kJ/h")
print(f"    Ecart QC (Delphi-Python)            = {QC_REF-QC_PY:>12,.1f} kJ/h ({(QC_REF/QC_PY-1)*100:+.2f}%)")
print(f"")
print(f"    HYPOTHESE 1: H[1]-H[0] identique, seul H[0]-h[0] differe =>")
print(f"      H[0]-h[0]_Delphi = {HH_delphi_h1:>8.2f} kJ/kmol  (Python: {H_PY[0]-h_PY[0]:.2f})")
print(f"      Ecart ΔHvap_Delphi - Python       = {HH_delphi_h1-(H_PY[0]-h_PY[0]):>+8.2f} kJ/kmol  ({(HH_delphi_h1/(H_PY[0]-h_PY[0])-1)*100:+.2f}%)")
print(f"")
print(f"    HYPOTHESE 2: H[0]-h[0] identique, seul H[1]-H[0] differe =>")
print(f"      H[1]-H[0]_Delphi  = {HH_delphi_h2:>8.2f} kJ/kmol  (Python: {H_PY[1]-H_PY[0]:.2f})")
print(f"      Ecart              = {HH_delphi_h2-(H_PY[1]-H_PY[0]):>+8.2f} kJ/kmol  ({(HH_delphi_h2/(H_PY[1]-H_PY[0])-1)*100:+.2f}%)")
print(f"")
print(f"  INTERPRETATION:")
print(f"    Le Terme B domine QC a {B_py/QC_PY*100:.0f}%. Si l ecart est dans le Terme B:")
print(f"    => ΔHvap Watson pour le condenseur est sous-estime de ~{(HH_delphi_h1/(H_PY[0]-h_PY[0])-1)*100:.1f}%")
print(f"    => correspond a ΔHvap trop bas de {HH_delphi_h1-(H_PY[0]-h_PY[0]):+.1f} kJ/kmol")
print(f"    Si l ecart est dans le Terme A:")
print(f"    => H[1]-H[0] (refroidissement vapeur) serait sous-estime de {(HH_delphi_h2/(H_PY[1]-H_PY[0])-1)*100:.1f}%")

# =============================================================================
# PART 5 - Phi (flux enthalpique montant) dans chaque section
# =============================================================================
print("\n" + "=" * 70)
print("PHI = net enthalpy flux montant (kJ/h) par section")
print("=" * 70)
print("""
phi[j] = V[j+1]*H[j+1] - L[j]*h[j]  [flux net montant entre j et j+1]

Pour condenseur PARTIEL: distillat D quitte en vapeur avec H[0]
    phi_rectif = D*H[0] + QC   [bilan sur le condenseur: V1*H1 = L0*h0 + D*H0 + QC]

Les alimentations SOUSTRAIENT de phi:
    phi_AB     = phi_rectif - VFC*H_FC
    phi_BC     = phi_AB - LFC*h_FC
    phi_CD     = phi_BC - VFH*H_FH
    phi_strip  = phi_CD - LFH*h_FH
    phi_strip  = QR - B*h[N-1]   [bilan rebouilleur]
""")

# Enthalpies alimentations
xFC_py = col.LFC / col.LFC_total
yFC_py = col.VFC / col.VFC_total
xFH_py = col.LFH / col.LFH_total
yFH_py = col.VFH / col.VFH_total

h_FC_py = col.eos.pure_component_enthalpy(FC_T, P_Pa, xFC_py, "liquid")
H_FC_py = col.eos.mixture_enthalpy(FC_T, P_Pa, yFC_py, "vapor")
h_FH_py = col.eos.pure_component_enthalpy(FH_T, P_Pa, xFH_py, "liquid")
H_FH_py = col.eos.mixture_enthalpy(FH_T, P_Pa, yFH_py, "vapor")

# Phi cumule (condenseur PARTIEL: D*H[0])
phi_rectif = D_val * H_PY[0] + QC_PY
phi_AB     = phi_rectif - VFC_total * H_FC_py
phi_BC     = phi_AB     - LFC_total * h_FC_py
phi_CD     = phi_BC     - VFH_total * H_FH_py
phi_strip  = phi_CD     - LFH_total * h_FH_py
phi_rebck  = QR_PY - B_REF * h_PY[N-1]

# Phi direct
phi_direct = np.array([V_PY[j+1]*H_PY[j+1] - L_PY[j]*h_PY[j] for j in range(N-1)])

print(f"  Section            phi_cumul (kJ/h)    phi_direct (kJ/h)   ecart")
print(f"  {'-'*65}")
sections = [
    ("rectifying (j=0..3)",  phi_rectif, phi_direct[0]),
    ("after VFC (j=4)",      phi_AB,     phi_direct[4]),
    ("mid-col (j=5..19)",    phi_BC,     phi_direct[5]),
    ("after VFH (j=20)",     phi_CD,     phi_direct[20]),
    ("stripping (j=21..28)", phi_strip,  phi_direct[21]),
]
for name, pc, pd in sections:
    print(f"  {name:<25s} {pc:>16,.1f}   {pd:>16,.1f}   {pc-pd:>+10.1f}")

print(f"\n  Closure: phi_strip (cumul) = {phi_strip:>14,.1f}")
print(f"           QR - B*h[B] (reb)  = {phi_rebck:>14,.1f}")
print(f"           Closure error       = {phi_strip - phi_rebck:>+14,.1f} kJ/h")

# =============================================================================
# PART 6 - Tableau enthalpies par etage
# =============================================================================
print("\n" + "=" * 110)
print("TABLEAU ENTHALPIES PAR ETAGE (Python converge)")
print("=" * 110)
print(f"{'j':>3}  {'T_ref':>7}  {'T_py':>7}  "
      f"{'h_py':>10}  {'H_py':>10}  {'H-h(lat)':>10}  "
      f"{'H[j+1]-H[j]':>13}  {'H[j+1]-h[j]':>13}  {'phi_dir':>13}")
print("-" * 110)
for j in range(N):
    lat  = H_PY[j] - h_PY[j]
    sens = H_PY[j+1] - H_PY[j]  if j < N-1 else float('nan')
    driv = H_PY[j+1] - h_PY[j]  if j < N-1 else float('nan')
    phi  = phi_direct[j]         if j < N-1 else float('nan')
    print(f"{j:3d}  {T_REF[j]:7.1f}  {T_PY[j]:7.1f}  "
          f"{h_PY[j]:10.1f}  {H_PY[j]:10.1f}  {lat:10.1f}  "
          f"{sens:13.1f}  {driv:13.1f}  {phi:13.1f}")

# =============================================================================
# PART 7 - Comparaison des quantites cles
# =============================================================================
print("\n" + "=" * 70)
print("COMPARAISON QUANTITES CLES (Python vs Delphi reference)")
print("=" * 70)
print(f"\n  {'Quantite':<42}  {'Delphi ref':>12}  {'Python':>12}  {'err%':>7}")
print(f"  {'-'*80}")

rows = [
    ("QC  (kJ/h)",                      QC_REF,           QC_PY),
    ("QR  (kJ/h)",                       QR_REF,           QR_PY),
    ("V[1]  (kmol/h)",                   V_REF[1],         V_PY[1]),
    ("L[0]  (kmol/h)",                   L_REF[0],         L_PY[0]),
    ("V[29] (kmol/h)",                   V_REF[29],        V_PY[29]),
    ("T[0]  (K)",                        T_REF[0],         T_PY[0]),
    ("T[1]  (K)",                        T_REF[1],         T_PY[1]),
    ("T[28] (K)",                        T_REF[28],        T_PY[28]),
    ("T[29] (K)",                        T_REF[29],        T_PY[29]),
    ("H[1]-H[0] (kJ/kmol) [refroid. v]", float('nan'),    H_PY[1]-H_PY[0]),
    ("H[0]-h[0] (kJ/kmol) [ΔHvap eff]",  float('nan'),    H_PY[0]-h_PY[0]),
    ("H[28]-h[28] (kJ/kmol) [latente]",  float('nan'),    H_PY[28]-h_PY[28]),
]
for label, ref_v, py_v in rows:
    if np.isnan(ref_v):
        print(f"  {label:<42}  {'(inconnu)':>12}  {py_v:>12,.2f}  {'---':>7}")
    else:
        err = (py_v/ref_v - 1.0)*100 if ref_v else float('nan')
        print(f"  {label:<42}  {ref_v:>12,.2f}  {py_v:>12,.2f}  {err:>+7.2f}%")

# Valeurs Watson directement (j=0)
print(f"\n  ΔHvap Watson implique Delphi (hyp. seul B differe) = {HH_delphi_h1:>10.2f} kJ/kmol")
print(f"  ΔHvap Watson Python (H[0]-h[0])                   = {H_PY[0]-h_PY[0]:>10.2f} kJ/kmol")
print(f"  Erreur relative                                    = {(HH_delphi_h1/(H_PY[0]-h_PY[0])-1)*100:>+9.2f}%")

# =============================================================================
# PART 8 - Analyse ΔHvap Watson composant par composant a T[0]
# =============================================================================
print("\n" + "=" * 70)
print("ANALYSE ΔHvap WATSON par composant a T[0]")
print("=" * 70)
print(f"  T[0] = {T_PY[0]:.2f} K, P = {P_bar:.2f} bar")
print(f"  (ΔHvap impacte h_py[0] = H_ig[0] - sum(x_i * ΔHvap_i))")
print(f"  Contribution totale au terme B = L[0]*sum(x_i*ΔHvap_i)")
print()

# Compute individual ΔHvap: h = H_ig - ΔHvap  =>  ΔHvap = H_ig - h
# H_ig at T[0] (limit P->0, vapor)
# Use pure_component_enthalpy with x=[1...0...] to get single component contribution
nc = l_conv.shape[0]
names = ["C1","C2","C3","iC4","nC4","iC5","nC5","C6","C7","C8","C9","C10","C11","C12","CO2","N2"]

# Get overall x[0] composition
x0 = x_PY[:, 0]
print(f"  {'Comp':>5}  {'x[0]':>8}  {'ΔHvap_i (approx)':>18}  {'x*ΔHvap_i':>14}")
print(f"  {'-'*55}")

total_watson = 0.0
for i in range(nc):
    if x0[i] > 1e-4:
        # Enthalpy pure liquid component i at T[0]
        x_pure = np.zeros(nc); x_pure[i] = 1.0
        h_pure_liq = col.eos.pure_component_enthalpy(T_PY[0], P_Pa, x_pure, "liquid")
        H_pure_vap = col.eos.mixture_enthalpy(T_PY[0], P_Pa, x_pure, "vapor")
        dHvap_i = H_pure_vap - h_pure_liq
        contrib = x0[i] * dHvap_i
        total_watson += contrib
        label = names[i] if i < len(names) else f"C{i}"
        print(f"  {label:>5}  {x0[i]:>8.4f}  {dHvap_i:>18.2f}  {contrib:>14.2f} kJ/kmol")

print(f"  {'SUM':>5}  {x0.sum():>8.4f}  {'--':>18}  {total_watson:>14.2f} kJ/kmol  [= H[0]-h[0] attendu]")
print(f"  calcul direct H[0]-h[0] = {H_PY[0]-h_PY[0]:.2f} kJ/kmol")
print(f"\n  NOTE: total_watson inclut H_dep_PR pure-composant mais PAS H_dep melange!")

# --- Decomposition rigoureuse des 3 termes de H[0]-h[0] ---
# h[0] = H_ig(T, x) - sum(x_i * ΔHvap_i)     <- pure_component_enthalpy, sans H_dep
# H[0] = H_ig(T, y) + H_dep_PR(T, P, y, vap) <- mixture_enthalpy, avec H_dep
# => H[0]-h[0] = [H_ig(y)-H_ig(x)] + H_dep_PR_vap + sum(x_i * ΔHvap_i_WATSON)
H_ig_x0  = col.eos.ideal_gas_enthalpy(T_PY[0], x_PY[:, 0])   # kJ/kmol
H_ig_y0  = col.eos.ideal_gas_enthalpy(T_PY[0], y_PY[:, 0])
H_dep_y0 = col.eos.enthalpy_departure(T_PY[0], P_Pa, y_PY[:, 0], "vapor")
Watson_mix = h_PY[0] - H_ig_x0    # = -sum(x_i * ΔHvap_i) (devrait etre negatif)
# => sum(x_i * ΔHvap_i) = H_ig_x0 - h_PY[0]
sum_watson_pure = H_ig_x0 - h_PY[0]
dHig_comp = H_ig_y0 - H_ig_x0     # effet de composition sur H_ig

print(f"\n  DECOMPOSITION H[0]-h[0] en 3 termes:")
print(f"    H_ig(y[0]) - H_ig(x[0])   = {dHig_comp:>+9.2f} kJ/kmol  [composition y vs x dans gaz ideal]"  )
print(f"    H_dep_PR(y[0], vapeur)     = {H_dep_y0:>+9.2f} kJ/kmol  [correction gaz reel PR, vapeur]")
print(f"    sum(x_i * ΔHvap_i) Watson = {sum_watson_pure:>+9.2f} kJ/kmol  [latente Watson liquide, AUCUN H_dep]")
print(f"    TOTAL = H[0]-h[0]          = {dHig_comp+H_dep_y0+sum_watson_pure:>+9.2f} kJ/kmol  [verification: {H_PY[0]-h_PY[0]:.2f}]")
print(f"\n  IMPORTANT: h[0] = pure_component_enthalpy() utilise ZERO H_dep PR pour le liquide!")
print(f"    => L erreur peut venir du H_dep_PR vapeur (si Delphi utilise SRK/BWRLS au lieu de PR)")
print(f"    => OU de ΔHvap_i Watson (coefficients DIPPR sous-estimes a T~261K)")
print(f"    => OU des deux a la fois")
print(f"\n  Si seul H_dep_PR erreur: H_dep_PR_Delphi = {H_dep_y0 + (HH_delphi_h1 - (H_PY[0]-h_PY[0])):>+9.2f} kJ/kmol"
      f"  (Python: {H_dep_y0:+.2f}, diff: {HH_delphi_h1-(H_PY[0]-h_PY[0]):+.2f})")
print(f"  Si seul Watson ΔHvap erreur: sum(x*ΔHvap)_Delphi = {sum_watson_pure + (HH_delphi_h1-(H_PY[0]-h_PY[0])):>9.2f} kJ/kmol"
      f"  (Python: {sum_watson_pure:.2f}, diff: {HH_delphi_h1-(H_PY[0]-h_PY[0]):+.2f})")

# =============================================================================
# PART 9 - Recap tous les cas
# =============================================================================
print("\n" + "=" * 70)
print("RESUME TOUS CAS  (condenseur partiel: QC = A + B)")
print("=" * 70)

QC_refs  = {1: 4_133_587.91, 2: 4_125_130.56, 3: 3_340_734.13}
QR_refs  = {1: 22_854_385.49, 2: 22_845_197.41, 3: 22_060_800.98}

print(f"\n  {'Cas':>3}  {'QC_ref':>14}  {'QC_py':>14}  {'err':>7}  "
      f"{'H[1]-H[0]':>12}  {'H[0]-h[0]':>12}  "
      f"{'ΔHvap_impl':>12}  {'err_ΔHvap':>10}")
print(f"  {'-'*100}")

for cn in [1, 2, 3]:
    c_c = Deethanizer()
    c_c.solve(case=cn, max_outer=20, tol=1.0, verbose=False)

    T_c  = c_c.T;  V_c = c_c.V;  L_c = c_c.L
    qc_c = abs(c_c.QC)

    l_c = c_c.l;  v_c = c_c.v
    x_c = np.zeros_like(l_c);  y_c = np.zeros_like(v_c)
    for j in range(N):
        Lj = l_c[:, j].sum();  Vj = v_c[:, j].sum()
        if Lj > 1e-10: x_c[:, j] = l_c[:, j] / Lj
        if Vj > 1e-10: y_c[:, j] = v_c[:, j] / Vj

    h0 = c_c.eos.pure_component_enthalpy(T_c[0], c_c.P, x_c[:, 0], "liquid")
    H0 = c_c.eos.mixture_enthalpy(T_c[0], c_c.P, y_c[:, 0], "vapor")
    H1 = c_c.eos.mixture_enthalpy(T_c[1], c_c.P, y_c[:, 1], "vapor")

    dH_sens = H1 - H0            # kJ/kmol  terme A unitaire
    dH_lat  = H0 - h0            # kJ/kmol  terme B unitaire

    A_c = V_c[1] * dH_sens
    B_c = L_c[0] * dH_lat

    # Implication Delphi: si seul terme B differe
    dH_lat_impl = (QC_refs[cn] - A_c) / L_c[0]
    err_lat = (dH_lat_impl / dH_lat - 1.0) * 100
    err_qc  = (qc_c / QC_refs[cn] - 1.0) * 100

    print(f"  {cn:>3}  {QC_refs[cn]:>14,.1f}  {qc_c:>14,.1f}  {err_qc:>+6.2f}%  "
          f"{dH_sens:>12.2f}  {dH_lat:>12.2f}  "
          f"{dH_lat_impl:>12.2f}  {err_lat:>+9.2f}%")

print(f"""
  COLONNES:
    H[1]-H[0]  : refroidissement sensible vapeur (T[1]->T[0]) en kJ/kmol
    H[0]-h[0]  : chaleur latente effective au condenseur en kJ/kmol (Python Watson)
    ΔHvap_impl : chaleur latente IMPLIQUEE par QC_Delphi si H[1]-H[0] identique
    err_ΔHvap  : erreur relative Watson vs ΔHvap implique Delphi

  CONCLUSION:
    V[1] et L[0] sont exacts => erreur QC = erreur purement dans les enthalpies.
    Le Terme B domine (~87% de QC).
    L ecart ~3-7% dans QC correspond a ~3-7% dans ΔHvap_Watson.
    Pistes: (1) coefficients Watson C2 sous-estimes a T~260K
            (2) correction PR departure differente (Delphi utilise peut-etre BWRLS ou SRK)
            (3) interactions binaires differentes dans le modele de melange
""")



import pandas as pd


def surplus_deficit(self):
    #print("self.df_intervals:::::!!!!!",self.df_intervals)
    # Group by 'IntervalName' and aggregate the values
    self.df_surplus_deficit = self.df_intervals.groupby('IntervalName').agg({
        'Tsup': 'first',  # Keep the first value
        'Tinf': 'first',  # Keep the first value
        'StreamName':  lambda x: list(x),  # Keep the first value
        'mCp': 'sum',  # Sum the 'mCp' values
        'StreamType': lambda x: list(x),  # Keep the first value
        'delta_T': 'first',  # Sum the 'delta_T' values
        'delta_H': 'sum'  # Sum the 'delta_H' values
    }).reset_index()

    # Sort by 'Tsup' in descending order
    #print("self.df_surplus_deficit==0===:::::",self.df_surplus_deficit)
    self.df_surplus_deficit = self.df_surplus_deficit.sort_values(by='Tsup', ascending=False)
    #print("self.df_surplus_deficit==1===:::::",self.df_surplus_deficit)
    self.df_surplus_deficit['cumulative_delta_H'] =self.df_surplus_deficit['delta_H'].cumsum()
    
    #print("self.df_surplus_deficit['cumulative_delta_H']=======::::::::",self.df_surplus_deficit['cumulative_delta_H'] )

    self.Heating_duty=pd.to_numeric(self.df_surplus_deficit['cumulative_delta_H'], errors='coerce').min()
    if self.Heating_duty>= 0:
        self.Heating_duty = 0
    else:
        self.Heating_duty = abs(self.Heating_duty)



    # Créer une ligne avec la valeur 0 pour 'cumulative_delta_H'
    self.cumulative_delta_H = pd.concat([pd.Series([0], name='cumulative_delta_H'), self.df_surplus_deficit['cumulative_delta_H']], ignore_index=True)

    # Ajouter la nouvelle ligne à self.Heating_duty
    self.cumulative_delta_H = pd.DataFrame(self.Heating_duty + self.cumulative_delta_H)

    # Concaténer les deux colonnes dans un nouveau DataFrame
    self.GCC = pd.concat([self.df_T_shifted, self.cumulative_delta_H], axis=1)

    # Récupérer la valeur de cumulative_delta_H correspondante
    self.Cooling_duty = self.GCC.loc[self.GCC['T_shifted'].idxmin(), 'cumulative_delta_H']

    # Récupérer la valeur de T_shifted correspondante à cumulative_delta_H nulle
    self.Pinch_Temperature = self.GCC.loc[self.GCC[self.GCC['cumulative_delta_H'] == 0].index, 'T_shifted'].values[0]
    




import pandas as pd
from HeatTransfer import PlateHeatTransfer

class Object:
    """
    Classe pour calculer le transfert de chaleur d'un corps parallélépipédique rectangulaire.
    
    Paramètres:
    -----------
    L : float
        Longueur en mètres (axe X)
    W : float
        Largeur en mètres (axe Y)
    H : float
        Hauteur en mètres (axe Z)
    Ta : float
        Température ambiante en °C
    faces_config : dict
        Configuration des faces avec structure:
        {
            'top': {'Tp': float, 'isolated': bool},
            'bottom': {'Tp': float, 'isolated': bool},
            'front': {'Tp': float, 'isolated': bool},
            'back': {'Tp': float, 'isolated': bool},
            'left': {'Tp': float, 'isolated': bool},
            'right': {'Tp': float, 'isolated': bool}
        }
    """
    
    def __init__(self, L, W, H, Ta, faces_config=None):
        self.L = L  # Longueur (X)
        self.W = W  # Largeur (Y)
        self.H = H  # Hauteur (Z)
        self.Ta = Ta
        
        # Configuration par défaut si non spécifiée
        if faces_config is None:
            default_Tp = Ta + 35
            faces_config = {
                'top': {'Tp': default_Tp, 'isolated': False},
                'bottom': {'Tp': default_Tp, 'isolated': False},
                'front': {'Tp': default_Tp, 'isolated': False},
                'back': {'Tp': default_Tp, 'isolated': False},
                'left': {'Tp': default_Tp, 'isolated': False},
                'right': {'Tp': default_Tp, 'isolated': False}
            }
        
        self.faces_config = faces_config
        self.results = {}
        self.df = None
        
    def calculate(self):
        """
        Calcule le transfert de chaleur pour toutes les faces.
        """
        # Face supérieure (horizontal_up)
        if not self.faces_config['top']['isolated']:
            self.results['top'] = PlateHeatTransfer.Object(
                orientation='horizontal_up',
                Tp=self.faces_config['top']['Tp'],
                Ta=self.Ta,
                W=self.W,
                L=self.L
            ).calculate()
        else:
            self.results['top'] = 0.0
        
        # Face inférieure (horizontal_down)
        if not self.faces_config['bottom']['isolated']:
            self.results['bottom'] = PlateHeatTransfer.Object(
                orientation='horizontal_down',
                Tp=self.faces_config['bottom']['Tp'],
                Ta=self.Ta,
                W=self.W,
                L=self.L
            ).calculate()
        else:
            self.results['bottom'] = 0.0
        
        # Face avant (vertical, W x H)
        if not self.faces_config['front']['isolated']:
            self.results['front'] = PlateHeatTransfer.Object(
                orientation='vertical',
                Tp=self.faces_config['front']['Tp'],
                Ta=self.Ta,
                W=self.W,
                H=self.H
            ).calculate()
        else:
            self.results['front'] = 0.0
        
        # Face arrière (vertical, W x H)
        if not self.faces_config['back']['isolated']:
            self.results['back'] = PlateHeatTransfer.Object(
                orientation='vertical',
                Tp=self.faces_config['back']['Tp'],
                Ta=self.Ta,
                W=self.W,
                H=self.H
            ).calculate()
        else:
            self.results['back'] = 0.0
        
        # Face gauche (vertical, L x H)
        if not self.faces_config['left']['isolated']:
            self.results['left'] = PlateHeatTransfer.Object(
                orientation='vertical',
                Tp=self.faces_config['left']['Tp'],
                Ta=self.Ta,
                W=self.L,
                H=self.H
            ).calculate()
        else:
            self.results['left'] = 0.0
        
        # Face droite (vertical, L x H)
        if not self.faces_config['right']['isolated']:
            self.results['right'] = PlateHeatTransfer.Object(
                orientation='vertical',
                Tp=self.faces_config['right']['Tp'],
                Ta=self.Ta,
                W=self.L,
                H=self.H
            ).calculate()
        else:
            self.results['right'] = 0.0
        
        # Créer le DataFrame
        self._create_dataframe()
        
        return self.results
    
    def _create_dataframe(self):
        """
        Crée un DataFrame pandas avec les résultats.
        """
        data = []
        
        for face_name, heat_transfer in self.results.items():
            face_config = self.faces_config[face_name]
            
            # Calcul de la surface
            if face_name in ['top', 'bottom']:
                surface = self.L * self.W
            elif face_name in ['front', 'back']:
                surface = self.W * self.H
            else:  # left, right
                surface = self.L * self.H
            
            data.append({
                'Face': face_name,
                'Orientation': self._get_orientation(face_name),
                'Surface (m²)': round(surface, 4),
                'Tp (°C)': face_config['Tp'],
                'Ta (°C)': self.Ta,
                'ΔT (°C)': face_config['Tp'] - self.Ta,
                'Isolated': face_config['isolated'],
                'Heat Transfer (W)': round(heat_transfer, 2),
                'Heat Flux (W/m²)': round(heat_transfer / surface, 2) if surface > 0 else 0
            })
        
        # Ajouter une ligne pour le total
        total_heat = sum(self.results.values())
        total_surface = 2 * (self.L * self.W + self.W * self.H + self.L * self.H)
        
        data.append({
            'Face': 'TOTAL',
            'Orientation': '-',
            'Surface (m²)': round(total_surface, 4),
            'Tp (°C)': '-',
            'Ta (°C)': self.Ta,
            'ΔT (°C)': '-',
            'Isolated': '-',
            'Heat Transfer (W)': round(total_heat, 2),
            'Heat Flux (W/m²)': round(total_heat / total_surface, 2) if total_surface > 0 else 0
        })
        
        self.df = pd.DataFrame(data)
    
    def _get_orientation(self, face_name):
        """
        Retourne l'orientation de la face.
        """
        orientations = {
            'top': 'Horizontal (up)',
            'bottom': 'Horizontal (down)',
            'front': 'Vertical',
            'back': 'Vertical',
            'left': 'Vertical',
            'right': 'Vertical'
        }
        return orientations.get(face_name, 'Unknown')
    
    def get_total_heat_transfer(self):
        """
        Retourne le transfert de chaleur total.
        """
        return sum(self.results.values())
    
    def print_summary(self):
        """
        Affiche un résumé des résultats.
        """
        if self.df is not None:
            print("\n" + "="*80)
            print(f"TRANSFERT DE CHALEUR - CORPS PARALLÉLÉPIPÉDIQUE")
            print("="*80)
            print(f"Dimensions: L={self.L}m x W={self.W}m x H={self.H}m")
            print(f"Température ambiante: {self.Ta}°C")
            print("="*80)
            print(self.df.to_string(index=False))
            print("="*80)



import pandas as pd
from .Two_flow_pinch_analysis import Two_flow_pinch_analysis  # Import the function
# === Fonction pour appliquer un échange de chaleur au-dessus du pinch ===
def apply_heat_exchange_above(one_HS_df, one_CS_df,remain_stream_list_above,remaining_recoverable_heat,used_streams,heat_exchangers):
    # print(f"-----initial------------- remain_stream_list_above_updated\n,{remain_stream_list_above}")

    #nonlocal remaining_recoverable_heat, used_streams  # Access the remaining recoverable heat and used streams
    hot_stream = one_HS_df.iloc[0].to_dict()
    cold_stream = one_CS_df.iloc[0].to_dict()

    # print(f"\n------------------- Applying Two_flow_pinch_analysis function above the pinch between HS_id={hot_stream['id']} and CS_id={cold_stream['id']} ---")
    results=Two_flow_pinch_analysis(hot_stream,cold_stream)
    # print("\n ---Two_flow_pinch_analysis : flux initiaux avant échange :\n", results["initial_fluxes"])
    # print("\n ---Two_flow_pinch_analysis : propriétés des flux récupérés :\n", results["exchanger"])
    # print("\n ---Two_flow_pinch_analysis : flux restants après récupération :\n", results["remaining_fluxes"])
    # print("\n---Two_flow_pinch_analysis : Utilité chaude :", results["hot_utility"])
    # print("\n---Two_flow_pinch_analysis : Utilité froide :", results["cold_utility"])
    # print("\n---Two_flow_pinch_analysis : Chaleur récupérée :", results["heat_recovered"])

    # === Étape 1 : suppression des lignes à remplacer ===
    remain_stream_list_above.drop(
        remain_stream_list_above[remain_stream_list_above['id'].isin(results["remaining_fluxes"]['id'])].index,
        inplace=True
    )
    # === Étape 2 : concaténation des nouvelles lignes ===
    remain_stream_list_above = pd.concat(
        [remain_stream_list_above, results["remaining_fluxes"]],
        ignore_index=True
    )


    one_HS_df = results["remaining_fluxes"][results["remaining_fluxes"]['id'] == hot_stream['id']]
    one_CS_df = results["remaining_fluxes"][results["remaining_fluxes"]['id'] == cold_stream['id']]

    heat_exchangers.append(results["exchanger"])
    remaining_recoverable_heat -= heat_exchangers[-1]['HeatExchanged']

 

    # print("\n3. ----------------------------Heat exchanger created above the pinch:")
    # print(heat_exchangers)
    # print("\n4. ---------------------------------------------------Updated remaining streams above the pinch:")
    # print(remain_stream_list_above)
    # print(f"one_HS_df----------------------------\n,{one_HS_df}\n")
    # print(f"one_CS_df----------------------------\n,{one_CS_df}\n")

    return one_HS_df, one_CS_df,remain_stream_list_above



import pandas as pd
from datetime import datetime
from sklearn.linear_model import LinearRegression
from scipy.stats import t
from sklearn.metrics import mean_squared_error
from sklearn.preprocessing import PolynomialFeatures



import numpy as np
#pip install statsmodels 
import statsmodels.api as sm

import docx
from docx.shared import Inches,RGBColor
import matplotlib.pyplot as plt

def docx_report(y_bl,y_report,df,df_report,conformite,conformite_report,table_incertitude,table_incertitude_report,df_savings,report_prediction,bl_prediction,start_baseline_period,end_baseline_period,start_reporting_period,end_reporting_period,site="****"):
    #pip install python-docx
    
    # Créer un nouveau document
    doc = docx.Document()

    # Ajouter un titre
    doc.add_heading('Rapport de mesure et vérification', 0)

    #Ajoutez un premier chapitre à votre rapport :
    doc.add_heading("1. Contexte du projet", level=1)
    # Ajouter un paragraphe de texte
    doc.add_paragraph("Ce projet s’inscrit dans le cadre des objectifs de réduction des consommations d’énergie sur le site de "+str(site)+ ". Il vise à réduire la consommation d’énergie liée au chauffage. La solution repose sur la mise en œuvre d’un système de pilotage des vannes thermostatiques en fonction des besoins thermiques des salles .")
    #doc.add_page_break()
    doc.add_heading("2. Description de la ou des AAPE", level=1)
    doc.add_paragraph("L’AAPE (Action d’Amélioration de la Performance Energétique) consiste à réguler les températures pièce par pièce et heure par heure en fonction des plannings d’occupation et de l’occupation réelle en agissant sur des vannes connectées. L’algorithme de régulation apprend également du comportement thermique du bâtiment afin d’adapterau plus juste la mise en marche et l’arrêt du chauffage.")
    doc.add_paragraph("Comment l’AAPE agit sur les consommations ?")
    doc.add_paragraph("Cette AAPE réduit la consommation d’énergie des zones inoccupées et améliore le confort. Par rapport à un planning de chauffe traditionnel pour lequel toutes les zones d’un bâtiment sont chauffées de manière quasiment égale voire continue en journée, les températures de consignes sont ici adaptées zone par zone en fonction de leur occupation prévisionnelle, de l’occupation réelle mesurée et du comportement de la pièce. L’occupation réelle mesurée pemet de réaliser des réduits de consigne pour générer une économie au-delà du respect du planning de consigne défini initialement. ")
    doc.add_paragraph("Cette AAPE génère en moyenne une économie de 25% sur la consommation de gaz.")


    #doc.add_page_break()
    doc.add_heading("3. Option M&V choisie pour l’AAPE", level=1)
    doc.add_paragraph("L’option choisie est :")
    doc.add_paragraph("l’option B, si le gain envisagé ne concerne qu'un sous ensemble du site et non l’ensemble du site. Il apparaît plus judicieux d’isoler l’AAPE sur son périmètre et de mesurer les consommations de gaz liées au chauffage avec l’ensemble des paramètres influents.")
    doc.add_paragraph("l’option C, si le gain envisagé concerne l’ensemble du site avec un compteur général d'un fournisseur d'énergie. Sont utilisées, ici les relevés de consommation de gaz de l’ensemble du site ainsi que les mesures de l'ensemble des facteurs infulents .")

    #doc.add_page_break()
    doc.add_heading("4. Rappel de la période de référence", level=1)
    doc.add_paragraph("La période de référence est du "+str(start_baseline_period)+" au "+str(end_baseline_period))
    #tracer la consommation de la periode de référence
    fig, ax = plt.subplots()
    ax.plot(y_bl.index, y_bl.iloc[:, 0], label=y_bl.columns[0])
    ax.set_xlabel('Date')
    ax.set_ylabel(y_bl.columns[0])
    ax.legend(loc='upper right')
    fig.savefig('conso_ref.png')
    doc.add_picture('conso_ref.png', width=Inches(6))
    # Ajouter une légende ou un titre à l'image
    caption = doc.add_paragraph('Figure 1. Consommation de la période de référence')
    caption.alignment = 1  # Aligner le texte au centre
    caption.bold = True  # Mettre en gras le texte de la légende

        ############ Créer une table avec la même forme que le dataframe#################
    y_bl_describe=y_bl.describe()
    y_bl_describe.loc["sum"]=y_bl.sum()
    y_bl_describe=y_bl_describe.round(2)
    table = doc.add_table(rows=y_bl_describe.shape[0]+1, cols=y_bl_describe.shape[1]+1)
    # Ajouter l'en-tête de colonne
    for j in range(y_bl_describe.shape[-1]):
        table.cell(0,j+1).text = y_bl_describe.columns[j]
    # Ajouter l'en-tête de ligne
    table.cell(0,0).text = 'Index'
    # Ajouter les données du dataframe
    for i in range(y_bl_describe.shape[0]):
        table.cell(i+1,0).text = str(y_bl_describe.index[i])
        for j in range(y_bl_describe.shape[-1]):
            table.cell(i+1,j+1).text = str(y_bl_describe.iloc[i,j])
    table.style = 'Colorful List Accent 1'
    ############################################################################

    #doc.add_page_break()
    doc.add_heading("5. La période de suivi", level=1)
    doc.add_paragraph("La période de suivi de ce rapport est du "+str(start_reporting_period)+" au "+str(end_reporting_period))
    
    fig, ax = plt.subplots()
    ax.plot(y_report.index, y_report.iloc[:, 0],label=y_report.columns[0])
    ax.set_xlabel('Date')
    ax.set_ylabel(y_report.columns[0])
    ax.legend(loc='upper right')

    fig.savefig('conso_suivi.png')
    doc.add_picture('conso_suivi.png', width=Inches(6))
    # Ajouter une légende ou un titre à l'image
    caption = doc.add_paragraph('Figure 2. Consommation de la période de suivi')
    caption.alignment = 1  # Aligner le texte au centre
    caption.bold = True  # Mettre en gras le texte de la légende

    ############ Créer une table avec la même forme que le dataframe#################
    y_report_describe=y_report.describe()
    y_report_describe.loc["sum"]=y_report.sum()
    y_report_describe=y_report_describe.round(2)
    table = doc.add_table(rows=y_report_describe.shape[0]+1, cols=y_report_describe.shape[1]+1)
    # Ajouter l'en-tête de colonne
    for j in range(y_report_describe.shape[-1]):
        table.cell(0,j+1).text = y_report_describe.columns[j]
    # Ajouter l'en-tête de ligne
    table.cell(0,0).text = 'Index'
    # Ajouter les données du dataframe
    for i in range(y_report_describe.shape[0]):
        table.cell(i+1,0).text = str(y_report_describe.index[i])
        for j in range(y_report_describe.shape[-1]):
            table.cell(i+1,j+1).text = str(y_report_describe.iloc[i,j])
    table.style = 'Colorful List Accent 1'
    ############################################################################
     
     
    #doc.add_page_break()
    doc.add_heading("6. Modèle de calcul des économies d'énergie", level=1)
    doc.add_heading("6.1. Modèle ANTE", level=2)
    doc.add_paragraph("Nous proposons ici d'évaluer un modèle sur la base la pèriode de référence qui sera appliqué sur la période de suivi pour le calcul des économies")

        ############ Créer une table avec la même forme que le dataframe#################
    table = doc.add_table(rows=df.shape[0]+1, cols=df.shape[1]+1)
    # Ajouter l'en-tête de colonne
    for j in range(df.shape[-1]):
        table.cell(0,j+1).text = df.columns[j]
    # Ajouter l'en-tête de ligne
    table.cell(0,0).text = 'Index'
    # Ajouter les données du dataframe
    for i in range(df.shape[0]):
        table.cell(i+1,0).text = str(df.index[i])
        for j in range(df.shape[-1]):
            table.cell(i+1,j+1).text = str(df.iloc[i,j])
    table.style = 'Colorful List Accent 1'
    ############################################################################
    doc.add_heading("6.1.1. conformité du Modèle ANTE", level=3)
    ############ Créer une table avec la même forme que le dataframe#################
    table = doc.add_table(rows=conformite.shape[0]+1, cols=conformite.shape[1]+1)
    # Ajouter l'en-tête de colonne
    for j in range(conformite.shape[-1]):
        table.cell(0,j+1).text = conformite.columns[j]
    # Ajouter l'en-tête de ligne
    table.cell(0,0).text = 'Index'
    # Ajouter les données du dataframe
    for i in range(conformite.shape[0]):
        table.cell(i+1,0).text = str(conformite.index[i])
        for j in range(conformite.shape[-1]):
            table.cell(i+1,j+1).text = str(conformite.iloc[i,j])
    table.style = 'Colorful List Accent 1'
    ############################################################################

    doc.add_heading("6.1.2. Incertitude du Modèle ANTE", level=3) 
        ############ Créer une table avec la même forme que le dataframe#################
    table = doc.add_table(rows=table_incertitude.shape[0]+1, cols=table_incertitude.shape[1]+1)
    # Ajouter l'en-tête de colonne
    for j in range(table_incertitude.shape[-1]):
        table.cell(0,j+1).text = table_incertitude.columns[j]
    # Ajouter l'en-tête de ligne
    table.cell(0,0).text = 'Index'
    # Ajouter les données du dataframe
    for i in range(table_incertitude.shape[0]):
        table.cell(i+1,0).text = str(table_incertitude.index[i])
        for j in range(table_incertitude.shape[-1]):
            table.cell(i+1,j+1).text = str(table_incertitude.iloc[i,j])
    table.style = 'Colorful List Accent 1'
    ############################################################################


  

    doc.add_heading("6.2. Modèle POST", level=2)
    doc.add_paragraph("Nous proposons ici d'évaluer un modèle sur la base la pèriode de suivi qui sera appliqué sur la période de référence dans le cas ou les données de la période de référence ne sont pas suffisantes pour établir un modèle valide")

        ############ Créer une table avec la même forme que le dataframe#################
    table = doc.add_table(rows=df_report.shape[0]+1, cols=df_report.shape[1]+1)
    # Ajouter l'en-tête de colonne
    for j in range(df_report.shape[-1]):
        table.cell(0,j+1).text = df_report.columns[j]
    # Ajouter l'en-tête de ligne
    table.cell(0,0).text = 'Index'
    # Ajouter les données du dataframe
    for i in range(df_report.shape[0]):
        table.cell(i+1,0).text = str(df_report.index[i])
        for j in range(df_report.shape[-1]):
            table.cell(i+1,j+1).text = str(df_report.iloc[i,j])
    table.style = 'Colorful List Accent 1'
    ############################################################################
    doc.add_heading("6.2.1. conformité du Modèle POST", level=3)
           ############ Créer une table avec la même forme que le dataframe#################
    table = doc.add_table(rows=conformite_report.shape[0]+1, cols=conformite_report.shape[1]+1)
    # Ajouter l'en-tête de colonne
    for j in range(conformite_report.shape[-1]):
        table.cell(0,j+1).text = conformite_report.columns[j]
    # Ajouter l'en-tête de ligne
    table.cell(0,0).text = 'Index'
    # Ajouter les données du dataframe
    for i in range(conformite_report.shape[0]):
        table.cell(i+1,0).text = str(conformite_report.index[i])
        for j in range(conformite_report.shape[-1]):
            table.cell(i+1,j+1).text = str(conformite_report.iloc[i,j])
    table.style = 'Colorful List Accent 1'
    ############################################################################

    doc.add_heading("6.2.2. Incertitude du Modèle POST", level=3)
        ############ Créer une table avec la même forme que le dataframe#################
    table = doc.add_table(rows=table_incertitude_report.shape[0]+1, cols=table_incertitude_report.shape[1]+1)
    # Ajouter l'en-tête de colonne
    for j in range(table_incertitude_report.shape[-1]):
        table.cell(0,j+1).text = table_incertitude_report.columns[j]
    # Ajouter l'en-tête de ligne
    table.cell(0,0).text = 'Index'
    # Ajouter les données du dataframe
    for i in range(table_incertitude_report.shape[0]):
        table.cell(i+1,0).text = str(table_incertitude_report.index[i])
        for j in range(table_incertitude_report.shape[-1]):
            table.cell(i+1,j+1).text = str(table_incertitude_report.iloc[i,j])
    table.style = 'Colorful List Accent 1'
    ############################################################################


    doc.add_heading("7. Calcul des économies d'énergie : option ANTE-POST et option POST-ANTE", level=1)
              ############ Créer une table avec la même forme que le dataframe#################
    table = doc.add_table(rows=df_savings.shape[0]+1, cols=df_savings.shape[1]+1)
  
  
            

    # Ajouter l'en-tête de colonne
    for j in range(df_savings.shape[-1]):
        table.cell(0,j+1).text = df_savings.columns[j]
    # Ajouter l'en-tête de ligne
    table.cell(0,0).text = 'Méthode de calcul'
    # Ajouter les données du dataframe
    for i in range(df_savings.shape[0]):
        table.cell(i+1,0).text = str(df_savings.index[i])
        for j in range(df_savings.shape[-1]):
            table.cell(i+1,j+1).text = str(df_savings.iloc[i,j])
    # Adding style to a table
    table.style = 'Colorful List Accent 1'
    ############################################################################


    #Economies ANTE-POST###############""
    fig, ax = plt.subplots(figsize=(8, 6))
    ax.plot(y_bl.index, y_bl.iloc[:, 0],label="relevé base de référence")
    ax.plot(y_report.index, y_report.iloc[:, 0],label="relevé période de suivi")
    ax.plot(report_prediction.index, report_prediction.iloc[:, 0],label="modèle IPMVP")

    ax.set_xlabel('Date')
    ax.set_ylabel(y_bl.columns[0])
    ax.legend(loc='upper right')

    fig.savefig('eco-ANTE-POST.png')
    doc.add_picture('eco-ANTE-POST.png', width=Inches(6))
    # Ajouter une légende ou un titre à l'image
    caption = doc.add_paragraph('Figure 3. Application du modèle sur la période POST')
    caption.alignment = 1  # Aligner le texte au centre
    caption.bold = True  # Mettre en gras le texte de la légende
    #########################""""

    #Economies POST-ANTE###############""
    fig, ax = plt.subplots(figsize=(8, 6))
    ax.plot(y_bl.index, y_bl.iloc[:, 0], label='relevé base de référence')
    ax.plot(y_report.index, y_report.iloc[:, 0], label='relevé période de suivi')
    ax.plot(bl_prediction.index, bl_prediction.iloc[:, 0],label="modèle IPMVP")
    ax.set_xlabel('Date')
    ax.set_ylabel(y_bl.columns[0])
    ax.legend(loc='upper right')

    fig.savefig('eco-POST-ANTE.png')
    doc.add_picture('eco-POST-ANTE.png', width=Inches(6))
    # Ajouter une légende ou un titre à l'image
    caption = doc.add_paragraph('Figure 4. Application du modèle sur la période ANTE')
    caption.alignment = 1  # Aligner le texte au centre
    caption.bold = True  # Mettre en gras le texte de la légende
    #########################""""

    #Ajoutez un deuxième chapitre à votre rapport :
    # doc.add_page_break()
    # doc.add_heading("Chapitre 2", level=1)

      # ajouter un nouveau style Equation
    # equation_style = doc.styles.add_style('Equation', docx.enum.style.WD_STYLE_TYPE.PARAGRAPH)
    # equation_style.base_style = doc.styles['Normal']
    # equation_style.font.size = docx.shared.Pt(12)
    # equation_style.font.name = 'Cambria Math'

    # ajouter un paragraphe contenant une équation avec des exposants
    # ajouter un paragraphe contenant l'équation
    # eq = doc.add_paragraph('', style='Equation')
    # eq.add_run('a').font.superscript = True
    # eq.add_run('x').font.subscript = True
    # eq.add_run('² + b = 0').font.subscript = True



    
    # Enregistrer le document
    
    doc.save(str(site)+'_rapport_M&V_'+datetime.now().strftime("%Y-%m-%d %H%M%S")+'.docx')

def creation_plynome(X,degree=1):
    if degree==1:
        X=X

    else: 
        # créer un objet PolynomialFeatures avec un degré de 2
        poly = PolynomialFeatures(degree=degree, include_bias=False)
        # transformer les données du DataFrame en un ensemble de données polynomiales de degré 2
        X_poly = poly.fit_transform(X)

        # Obtenir les noms des colonnes pour le nouveau dataframe
        col_names = poly.get_feature_names(X.columns)
        # Créer un nouveau dataframe avec les noms de colonnes
        X_poly = pd.DataFrame(X_poly, columns=col_names,index=X.index)
        # afficher les données transformées
        X=X_poly
    print("X_poly=",X)

    return X



# supp les val aber.
def drop_outliers (df,seuil_z_scores):
    # Select only the numeric columns
    df = df.select_dtypes(include=np.number)    
    # Calculate the z-score for the numeric columns
    z_scores = (df - df.mean()) / df.std()
    print("z_scores----------------------------",z_scores.max())
    # Get the location of the outliers
    outlier_location = np.argwhere((z_scores > seuil_z_scores).values)
    #valeur à supprimer
    df_out = df[(z_scores >= seuil_z_scores).any(axis=1)]
    print(df_out.index,type(df_out.index))

    if df_out.empty or df_out.isnull().all(axis=1).any():
        pass
    else:
        print("valeurs à supprimer",df_out,outlier_location )

    # # Exclude rows where any z-score is greater than 3
    df = df[(z_scores < seuil_z_scores).all(axis=1)]
    #print(df)
    return df,df_out


def conformite_ipmvp(r2,cv_remse,stat_t,stat_t_normale95): #,,
    conformite=[]
    if r2>=0.75:
        conformite.append([r2,True])
    else:
        conformite.append([r2,False])
    if cv_remse<=0.2:
        conformite.append([cv_remse,True])
    else:
        conformite.append([cv_remse,False])

    conformite=(pd.DataFrame(conformite,index=["r2","cv_remse"],columns=["valeur","conformité IPMVP"]))
    stat_t = stat_t.rename(index={stat_t.index[0]: 'valeur'})
    stat_t_T=stat_t.T
    stat_t_T["conformité IPMVP"] = stat_t_T["valeur"].apply(lambda x: True if (x is not None and float(x) >= stat_t_normale95) else False) 
    
    conformite=pd.concat([conformite,stat_t_T])
    # print(conformite)

    return conformite

# stat_t=pd.DataFrame([[-0.072971,12.51099,None,None]],columns=["stat_t_const",  "stat_t_DJU",  "stat_t_x2" , "stat_t_x3"],index=["valeur"])    
# conformite_ipmvp(0.9,0.1,stat_t)
def regression_model(X_bl,y_bl,approache):
    #approache="ANTE-POST"
    # Modeling :
    model = LinearRegression()
    model.fit(X_bl,y_bl)
    y_pred = model.predict(X_bl) # predicted consumption
    
    n = len(y_bl) # nombre de population
    p=X_bl.shape[1] #nb variables explicatives

    # Calcul du R2
    r2 = model.score(X_bl, y_bl)
    #print('R2:', r2)

    # Affichage des coefficients et l'intercept
    coefficients = []
    for p_ in range(p):
        coefficients.append(model.coef_[0, p_])
    
    const = model.intercept_[0]



    # Calcul du RMSE
    mse = mean_squared_error(y_bl, y_pred, squared=True)
    ddof=max(n-p-1,0) # le degrès de liberté (comme celui d'excel)
    rmse=(mse*((n)/ddof))**0.5 #comme celui d'excel

    stat_t_normale95=t.ppf(1-0.05/2, ddof)
    print("statistique ************************************* loi normale",ddof,stat_t_normale95)
    
   
    #calcul du CV_RMSE
    cv_rmse=rmse/y_bl.mean()[0]

    ######################calculs d'incertitude##################################################
    niveau_confiance=0.8
    gamma=(1-niveau_confiance)/2
    stat_t_normale=t.ppf(niveau_confiance+gamma, ddof)
    
    print("statistique t loi normale",ddof,stat_t_normale)
    precision_absolue=stat_t_normale*rmse
    precision_relative=precision_absolue/y_bl.mean()[0]


    table_incertitude=pd.DataFrame({"valeurs":[gamma,niveau_confiance,stat_t_normale,rmse,precision_absolue,precision_relative]},index=["gamma","niveau_confiance","stat_t_normale","Erreur type (rmse)","precision_absolue +/-","precision_relative"]).round(2)
    print(table_incertitude)
    #############################################################################################################################################################################################################
   

    # calcul des erreurs standard...................................................
    # Calculer les erreurs types des coefficients
    # add a constant term to the input data for statsmodels
    X_bl_withConst = sm.add_constant(X_bl)
    # fit a linear regression model using statsmodels
    model_sm = sm.OLS(y_bl, X_bl_withConst).fit()
    # extract the standard errors of the coefficients from the statsmodels model
    serr = pd.DataFrame(model_sm.bse).T
    serr = serr.rename(index={0: approache})
    #serr=serr.add_prefix("SE_")

    # print the standard errors
    print("serr====================",serr)

    const_coef=(pd.DataFrame([np.concatenate(([const], coefficients))],columns=np.concatenate((['const'], X_bl.columns)).tolist()).rename(index={0: approache})).astype(float)
    print("const_coef=========================",const_coef)

    #calcul de la statistique t de chaque coef
    #stat_t=const_coef.astype(float).div(serr.astype(float))
   
    print(list(serr.columns),list(const_coef.columns))
    stat_t=const_coef.div(serr)
    
    #print("stat_t===============",stat_t)

    stat_t=stat_t.add_prefix("stat_t_")

    print("stat_t===============",stat_t)
    
    #arrondir
    serr=serr.round(1)
    stat_t=stat_t.round(1)
    r2=round(r2,2)
    rmse=round(rmse,1)
    cv_rmse=round(cv_rmse,2)
    ddof=round(ddof,0)
    const_coef=const_coef.round(10)

   
    

    # Modèle y_predicted_arr = modèle y avec arrondis [const,DJU,planning]:
    df=pd.DataFrame([[r2,rmse,cv_rmse,ddof]], columns=["r2","rmse","cv_rmse","ddof"], index=[approache])
  
    df=pd.merge( const_coef.add_prefix("coef_"),df, left_index=True, right_index=True)
    df=pd.merge(df, serr.add_prefix("serr_"), left_index=True, right_index=True)
    df=pd.merge(df,stat_t, left_index=True, right_index=True).T

    y_pred=pd.DataFrame(y_pred)   
    y_pred = y_pred.set_index(y_bl.index)  

    conformite=conformite_ipmvp(r2,cv_rmse,stat_t,stat_t_normale95)

    return model,y_pred,df,conformite,table_incertitude


def Mathematical_Models(y,X,start_baseline_period,end_baseline_period,start_reporting_period,end_reporting_period,print_report=False,seuil_z_scores=8,degree=1,site="****"):
    #remettre les données dans l'ordre
    y=pd.DataFrame(y).sort_index(axis=0, ascending=True, inplace=False, kind="quicksort")
    X=pd.DataFrame(X).sort_index(axis=0, ascending=True, inplace=False, kind="quicksort")
    y,y_out=drop_outliers(y,seuil_z_scores)
    #X,X_out=drop_outliers(X,3)
    X= X.drop(y_out.index, axis=0)

    ###################Ajouter etude des corrélations pol
    X=creation_plynome(X,degree=degree)


    #calculer de delta t
    dt = y.index.to_series().diff().astype('timedelta64[h]')
    curent_delta_time=dt.median()
    print("curent_delta_time (heures)",curent_delta_time)


    # if end_reporting_period is None :
    #     end_reporting_period = df.tail(1).index.strftime("%Y/%m/%d")[0]
    #     print("the end of the reporting period is the current date",end_reporting_period)

    #le jeu de données de la periode de référence
    X_bl = X.loc[(X.index >= start_baseline_period) & (X.index <= end_baseline_period)]
    #print("X base line", X_bl.head)

    y_bl = y.loc[(y.index >= start_baseline_period) & (y.index <= end_baseline_period)]
    print("y base line", y_bl)

    # jeu de données de la periode de suivi
    X_report = X.loc[(X.index >= start_reporting_period) & (X.index <= end_reporting_period)]
    y_report = y.loc[(y.index >= start_reporting_period) & (y.index <= end_reporting_period)]

    
##########################################################################################################################""
    # # Modeling :
   

    model_bl,y_pred,df,conformite,table_incertitude=regression_model(X_bl,y_bl,"ANTE-POST") 

    model_report,y_pred_report,df_report,conformite_report,table_incertitude_report=regression_model(X_report,y_report,"POST-ANTE") 

    report_prediction=model_bl.predict(X_report)
    sum_report_prediction= sum(report_prediction)[0]
    bl_prediction=model_report.predict(X_bl)
    sum_bl_prediction= sum(bl_prediction)[0]

    report_prediction=pd.DataFrame(report_prediction,index=X_report.index)
    print("report_prediction",report_prediction)

    bl_prediction=pd.DataFrame(bl_prediction,index=X_bl.index)
    print("bl_prediction",bl_prediction)

    sum_bl=y_bl.sum().values[0]
    sum_report=y_report.sum().values[0]

    savings_post=(sum_report_prediction-sum_report)/sum_report*100
    saving_ante=(-sum_bl_prediction+sum_bl)/sum_bl*100 #inversé pour avoir des éco positives

    df_savings=pd.DataFrame({"ANTE-POST":[sum_report,sum_report_prediction,savings_post],"POST-ANTE":[sum_bl,sum_bl_prediction,saving_ante]},index=["Relevé de consommation","Prédiction","pourcentage d'économie>0"]).round(2)
    
   
    if print_report==True:
        x=docx_report(y_bl,y_report,df,df_report,conformite,conformite_report,table_incertitude,table_incertitude_report,df_savings,report_prediction,bl_prediction,start_baseline_period,end_baseline_period,start_reporting_period,end_reporting_period,site=site)

    return  y_pred,df,conformite,table_incertitude,y_pred_report,df_report,conformite_report,table_incertitude_report,df_savings
   
  




from AHU.air_humide import air_humide
import pandas as pd
from AHU.AirPort.AirPort import AirPort
from ThermodynamicCycles.FluidPort.FluidPort import FluidPort  # Correction de l'import
import CoolProp.CoolProp as CP
import math

class HeatingCoilNUT:
    def __init__(self):
        # Ports air
        self.Inlet = AirPort()
        self.Outlet = AirPort()
        
        # Ajout du port eau
        self.Water_Inlet = FluidPort(fluid='Water')
        self.Water_Outlet = FluidPort(fluid='Water')
        
        # Paramètres généraux
        self.id = 2
        self.P_drop = 0  # Perte de charge air [Pa]
        
        # Paramètres côté air
        self.T_in = 0    # Température entrée air [°C]
        self.hi = 0      # Enthalpie entrée air [kJ/kg]
        self.wi = 12     # Humidité spécifique entrée [g/kg]
        self.F = 0       # Débit massique air humide [kg/s]
        self.F_dry = 0   # Débit massique air sec [kg/s]
        
        # Paramètres côté eau
        self.T_water_in = 80    # Température entrée eau [°C]
        self.T_water_out = 60   # Température sortie eau [°C]
        self.P_water = 3e5      # Pression eau [Pa]
        self.m_water = 0        # Débit massique eau [kg/s]
        
        # Paramètres échangeur
        self.S = 10             # Surface d'échange [m²]
        self.U = 50            # Coefficient d'échange global [W/m².K]
        self.NUT = 0           # Number of Transfer Units [-]
        self.effectiveness = 0  # Efficacité de l'échangeur [-]
        
        # Résultats
        self.Q_th = 0          # Puissance échangée [kW]
        self.To_target = 20    # Température de consigne sortie air [°C]
        self.ho = 0            # Enthalpie sortie air [kJ/kg]
        self.RH_out = 0        # Humidité relative sortie [%]
        
        self.df = None         # DataFrame résultats

    def calculate(self):
        # Connecteur air
        self.Outlet.P = self.Inlet.P - self.P_drop
        
        # Récupération conditions entrée air
        self.wi = self.Inlet.w
        self.hi = self.Inlet.h
        self.F = self.Inlet.F
        self.T_in = air_humide.Air_T_db(w=self.wi, h=self.hi, P=self.Inlet.P)
        self.F_dry = self.F/(1+self.wi/1000)  # [kg air sec/s]
        
        # Utilisation des propriétés du port d'eau
        self.T_water_in = self.Water_Inlet.T - 273.15  # Conversion K -> °C
        self.P_water = self.Water_Inlet.P
        self.m_water = self.Water_Inlet.F
        
        # Calcul propriétés eau
        cp_water = CP.PropsSI('CP0MASS', 'T', self.T_water_in + 273.15, 'P', self.P_water, 'Water')
        rho_water = CP.PropsSI('D', 'T', self.T_water_in + 273.15, 'P', self.P_water, 'Water')
        
        # Calcul des débits calorifiques
        C_water = self.m_water * cp_water    # [W/K]
        C_air = self.F_dry * 1006           # [W/K] (cp air ≈ 1006 J/kg.K)
        
        C_min = min(C_water, C_air)
        C_max = max(C_water, C_air)
        
        # Calcul NUT et efficacité
        self.NUT = self.U * self.S / C_min
        Cr = C_min/C_max
        
        # Efficacité pour échangeur à courants croisés non brassés
        #expenotiel en python
        
        self.effectiveness = 1 - math.exp(-self.NUT * (1 - math.exp(-Cr * self.NUT))/Cr)
        
        # Puissance maximale possible
        Q_max = C_min * (self.T_water_in - self.T_in)
        
        # Puissance réelle échangée
        self.Q_th = self.effectiveness * Q_max / 1000  # [kW]
        
        if self.To_target > self.T_in:
            # Calcul température sortie air
            delta_T = self.Q_th * 1000 / (self.F_dry * 1006)
            T_out = self.T_in + delta_T
            
            # Calcul propriétés air sortie
            self.ho = air_humide.Air_h(T_db=T_out, w=self.wi)
            self.RH_out = air_humide.Air_RH(
                Pv_sat=air_humide.Air_Pv_sat(T_out),
                w=self.wi,
                P=self.Outlet.P
            )
        else:
            self.ho = self.hi
            self.Q_th = 0
            
        # Connecteur sortie air
        self.Outlet.w = self.Inlet.w
        self.Outlet.h = self.ho
        self.Outlet.F = self.F_dry * (1 + self.Outlet.w/1000)
        self.Outlet.F_dry = self.F_dry
        
        # Mise à jour du port de sortie eau
        self.Water_Outlet.P = self.Water_Inlet.P  # Néglige les pertes de charge eau
        delta_h = self.Q_th * 1000 / self.m_water  # Variation d'enthalpie eau [J/kg]
        self.Water_Outlet.h = self.Water_Inlet.h - delta_h
        self.Water_Outlet.F = self.Water_Inlet.F

        # Mise à jour des propriétés du port eau
        self.Water_Outlet.calculate_properties()
        
        # Calcul T_water_out
        self.T_water_out = self.Water_Outlet.T - 273.15  # Conversion K -> °C
        
        # Mise à jour DataFrame résultats
        self.df = pd.DataFrame(
            data={
                'HeatingCoilNUT': [
                    self.id,
                    round(self.Outlet.T, 1),
                    round(self.Outlet.RH, 1),
                    round(self.Outlet.F, 3),
                    round(self.Outlet.F_dry, 3),
                    round(self.Outlet.P, 1),
                    round(self.Outlet.P / 100000, 1),
                    round(self.Outlet.h, 1),
                    round(self.Outlet.w, 3),
                    round(self.Outlet.Pv_sat, 1),
                    round(self.Q_th, 1),
                    round(self.NUT, 2),
                    round(self.effectiveness, 2),
                    round(self.T_water_out, 1)
                ]
            },
            index=[
                'ID', 'Outlet.T (C)', 'Outlet.RH (%)',
                'Outlet.F (kg/s)', 'Outlet.F_dry (kg/s)',
                'Outlet.P (Pa)', 'Outlet.P/10^5 (bar)',
                'Outlet.h (kJ/kg)', 'Outlet.w (g/kgdry)',
                'Outlet.Pv_sat (Pa)', 'Q_th (kW)',
                'NUT (-)', 'Effectiveness (-)',
                'T_water_out (C)'
            ]
        )